import { useState, useEffect, useRef } from "react";

export const useFetch = ({ url, token, dataSend, method = "GET" }) => {
    const isMounted = useRef(true);
    const [state, setState] = useState({
        data: null,
        loading: true,
        error: null,
    });

    useEffect(() => {
        return () => {
            isMounted.current = false;
        };
    }, []);

    useEffect(() => {
        setState({ data: null, loading: true, error: null });
        if (url) {
            fetch(url, {
                method: method,
                headers: {
                    Accept: "application/json, text/plain, */*",
                    "Content-Type": "application/json",
                    token: token,
                },
                body:
                    method === "GET"
                        ? null
                        : JSON.stringify({ data: dataSend }),
            })
                .then((resp) => resp.json())
                .then((data) => {
                    if (isMounted.current) {
                        setState({
                            loading: false,
                            error: data?.error ?? null,
                            data,
                        });
                    }
                })
                .catch(() => {
                    setState({
                        data: null,
                        loading: false,
                        error: "No se pudo cargar los datos",
                    });
                });
        }
    }, [dataSend, method, url]);

    return state;
};
