import { useState, useEffect, useRef } from "react";

export const useFetchData = ({url},{token}) => {
   
    const isMounted = useRef(true);
    const [state, setState] = useState({
        data: null,
        loading: true,
        error: null,
    });

    useEffect(() => {
        return () => {
            isMounted.current = false;
        };
    }, []);

    useEffect(() => {        

        //Definim la funció per obtenir dades
        const getData = () => {
            setState((prevState)=>({
                ...prevState,
                loading:true
            }))
            fetch(`${url}`,{
                method:"GET",
                headers: {
                    Accept: "application/json, text/plain, */*",
                    "Content-Type": "application/json",
                    "token":token,
                },               
            })
                .then((resp) => resp.json())
                .then(({data}) => { 
                    data.sort((a,b)=> a.object.IMEI-b.object.IMEI)
                    if (isMounted.current) {                              
                        setState({
                            loading: false,
                            error: null,
                            data 
                        });
                    }
                })
                .catch(() => {
                    setState({
                        data: null,
                        loading: false,
                        error: "Error data fetch",
                    });
                });
        };

        //Cridem la funció en intervals de 5,5 segons
        const interval = setInterval(getData, 5500);

        //Cridem la funció la primera vegada a 0 segons si hi ha token.
        !!token && getData();

        return () => {            
            clearInterval(interval)}; //Aixo es important reset interval
    }, [token, url]);

    return state;
};
