import { useState, useEffect, useRef } from "react";
import { pusher } from "../utils/pusher/pusherCon";

export const usePusher = ({ idEntity },  event ) => {
    const isMounted = useRef(true);
    const [state, setState] = useState({
        dataPusher: null,
        loadingPusher: true,
        errorPusher: null,
    });
    
    useEffect(() => {
        return () => {
            isMounted.current = false;
        };
    }, []);

    useEffect(() => {
        try {            
            
            const channel = pusher.subscribe(idEntity);
                        
            channel.bind(`${event}-${idEntity}`, function (data) {                
                setState({
                    dataPusher: data,
                    loadingPusher: false,
                    errorPusher: null,
                });
            });
            
            setState((prevState) => ({
                ...prevState,
                loading: false,
            }));
            
        } catch (error) {
            setState({
                dataPusher: null,
                loadingPusher: false,
                errorPusher: "Error al connectar amb el servei pusher",
            });
        }
        return () => {
            channel.unbind(`${event}-${idEntity}`);
            pusher.unsubscribe(idEntity);
        };
    }, [event, idEntity]);

    return state;
};
