/**
 * Hook que carrega l'usuari identificat i controla les seves funcions.
 * Controla l'actualització del token de Firebase, l'usuari i la funcio
 * de logout. Es poden cridar des de el context accecible des de qualsevol
 * lloc.
 */
import { useEffect, useState } from "react";
import firebase from "firebase/app";
import { mapUserData } from "../utils/auth/mapUserData";
import {
    removeUserCookie,
    setUserCookie,
    getUserFromCookie,
} from "../Hooks/userCookies";
// import { pusher } from "../utils/pusher/pusherCon";

let r = 0;

//Funció exportada del hook
const useUser = () => {
    const [user, setUser] = useState(undefined);

    //Controlar el logout de l'usuari
    const logout = async () => {
        // pusher.disconnect();
        return firebase
            .auth()
            .signOut()
            .then(() => {
                removeUserCookie();
                console.info("SignOut");
            })
            .catch((e) => {
                console.error("SignOut", e);
            });
    };

    useEffect(() => {
        // Firebase actualitza el testimoni d'identificació cada hora, això
        // assegura que l'estat de react i la galeta de
        // tots dos s'actualitzen mentre estigui en ús en navegador.
        const refresh = () => {
            console.log("Refrescar Token Firebase");
            firebase.auth().currentUser?.getIdToken(true);
        };

        //Cada 55minuts que refresqui el token si esta Chrome obert
        //Firebase ho fa cada 60minuts, s'ha de fer abans que Firebase.
        const interval = setInterval(refresh, 55 * 60 * 1000);

        //Si hi ha algun usuari sense expirar que l'agafi
        //i l'utilitzi per identificar-se, sino logout()
        //És podria fer sense la cookie, pero el onidTokenChanged
        //triga un segon en comprovar i fa un salt si es torna a entra un cop
        //loginat. En canvi, amb la cookie es directe. Sense el salt.
        const checkCookie = async () => {
            const userFromCookie = await getUserFromCookie();
            if (!userFromCookie) {
                logout();
            } else {
                setUser(userFromCookie);
            }
        };
        checkCookie();

        const cancelAuthListener = firebase
            .auth()
            .onIdTokenChanged(async (u) => {
                let q = Date.now();
                let y = q - r;                           
                if (y > 500) {
                    console.info("OnIdToKenChanged");
                    r = Date.now();

                    if (u) {
                        const userData = await mapUserData(u);
                        console.info("UpdateCookie:", Date());
                        setUserCookie(userData);
                        setUser(userData);
                    } else {
                        removeUserCookie();
                        setUser();
                        console.info("User not login");
                    }
                }
            });

        //Si desapareix el component que faci neteja
        return () => {
            console.log("limpiar effect");
            clearInterval(interval);
            cancelAuthListener();
        };
    }, []);

    return { user, setUser, logout };
};

export { useUser };
