// eslint-disable-next-line @next/next/no-img-element
import Modal from "react-bootstrap/Modal";
import "react-toastify/dist/ReactToastify.css";
import { convertDate } from "../../../utils/lib/dates";

const mapBoxToken = process.env.NEXT_PUBLIC_MAPBOX;

function Item({ par, src, width = 15, height = 15, name, value }) {
    return (
        <div className={`item ${par}`}>            
            <img
                alt={name}
                className="item-icon"
                src={src}
                width={width}
                height={height}
            />
            <div className="item-name">
                <strong>{name}</strong>
            </div>
            <div className="item-value">{value}</div>
        </div>
    );
}
export const ModalAmpliada = (props) => {
    const { show, setshow, title, data, ...otherProps } = props;
    const { geometry, properties, object } = data;
    

    const handleClose = () => {
        setshow(false);
    };
    

    return (
        <>
            <Modal
                id="serviceModal"
                {...otherProps}
                show={show}
                onHide={handleClose}
                backdrop="static"
                animation={false}
                size="lg"
                aria-labelledby="contained-modal-title-vcenter"
                scrollable
            >
                <Modal.Header closeButton>
                    <Modal.Title>{title}</Modal.Title>
                </Modal.Header>
                <Modal.Body>
                    <div className="list-items">
                        <Item
                            par={`impar`}
                            src={`/miscelanea/megaphone.svg`}
                            name={`Indicatiu`}
                            value={`${object?.name}`}
                        />
                        <Item
                            par={`par`}
                            src={`/miscelanea/clock.svg`}
                            name={`Marca temps`}
                            value={`${convertDate(properties?.timestamp)}h`}
                        />
                        <div className="separador" />
                        <Item
                            par={`impar`}
                            src={`/miscelanea/gps.svg`}
                            name={`Latitud`}
                            value={geometry?.coordinates[1]}
                        />
                        <Item
                            par={`par`}
                            src={`/miscelanea/gps.svg`}
                            name={`Longitud`}
                            value={geometry?.coordinates[0]}
                        />
                        <Item
                            par={`impar`}
                            src={`/miscelanea/altitude.svg`}
                            name={`Alçada`}
                            value={`${properties?.gps?.altitude} m`}
                        />
                        <Item
                            par={`par`}
                            src={`/miscelanea/angle.svg`}
                            name={`Angle`}
                            value={`${properties?.gps?.angle}°`}
                        />
                        <Item
                            par={`impar`}
                            src={`/miscelanea/precision.svg`}
                            name={`Precisió`}
                            value={`${properties?.gps?.precision}`}
                        />
                        <Item
                            par={`par`}
                            src={`/miscelanea/satellite.svg`}
                            name={`Satèl·lits`}
                            value={`${properties?.gps?.satellites}`}
                        />
                        <Item
                            par={`impar`}
                            src={`/miscelanea/battery2.svg`}
                            name={`Bateria GPS`}
                            value={`${properties?.ioElements?.battery_device}mA`}
                        />
                        <Item
                            par={`par`}
                            src={`/miscelanea/signal.svg`}
                            name={`Senyal`}
                            value={`${properties?.ioElements?.gsm_strengh}`}
                        />
                        <Item
                            par={`impar`}
                            src={`/miscelanea/type.svg`}
                            name={`Tipus GPS`}
                            value={`${properties?.type}`}
                        />
                        <Item
                            par={`par`}
                            src={`/miscelanea/factory.svg`}
                            name={`Marca GPS`}
                            value={`${object?.GPSdevice}`}
                        />
                        <Item
                            par={`impar`}
                            src={`/miscelanea/imei.svg`}
                            name={`IMEI`}
                            value={`${object?.IMEI}`}
                        />
                        <div className="separador" />
                        <Item
                            par={`impar`}
                            src={`/miscelanea/speed.svg`}
                            name={`Velocitat`}
                            value={`${properties?.gps?.speed} Km/h`}
                        />
                        <Item
                            par={`par`}
                            src={`/miscelanea/ignition.svg`}
                            name={`Encès`}
                            value={`${
                                properties?.ioElements?.ignition ? "Sí" : "No"
                            }`}
                        />
                        <Item
                            par={`impar`}
                            src={`/miscelanea/fuel.svg`}
                            name={`Carburant`}
                            value={`${properties?.ioElements?.fuel}%`}
                        />
                        <Item
                            par={`par`}
                            src={`/miscelanea/battery.svg`}
                            name={`Bateria`}
                            value={`${properties?.ioElements?.battery}mV`}
                        />
                        <Item
                            par={`impar`}
                            src={`/miscelanea/car.svg`}
                            name={`Marca`}
                            value={`${object?.marca}`}
                        />
                        <Item
                            par={`par`}
                            src={`/miscelanea/car.svg`}
                            name={`Model`}
                            value={`${object?.model}`}
                        />
                        <Item
                            par={`impar`}
                            src={`/miscelanea/plate.svg`}
                            name={`Matrícula`}
                            value={`${object?.plateNumber}`}
                        />
                        <Item
                            par={`par`}
                            src={`/miscelanea/imei.svg`}
                            name={`VIN`}
                            value={`${object?.VIN}`}
                        />
                    </div>
                </Modal.Body>
            </Modal>
            <style jsx global>{`
                .list-items {
                    font-size: 15px;
                    width: 100%;
                }
                .item {
                    display: grid;
                    grid-template-columns: 5% 30% 50%;
                    align-content: space-evenly;
                    justify-content: center;
                    align-items: baseline;
                    justify-items: stretch;
                    color: #444444;
                }
                .item.impar {
                    background: #007bff3e;
                }
                .separador {
                    margin: 15px 0px;
                    border-bottom: 1px solid grey;
                }
            `}</style>
        </>
    );
};
