import Modal from "react-bootstrap/Modal";
import { Col, Form } from "react-bootstrap";
import { toast} from "react-toastify";
import Select from "react-select";
import useUserContext from "../../../contexts/UserContext";
import "react-toastify/dist/ReactToastify.css";
import { GeneralModal } from "../../formulari/GeneralModal";
import { Button } from "../../globals/Button";
import { useEffect,useState } from "react";
import { useForm } from "../../../Hooks/useForm";
import ObjectsContext from "../../../contexts/ObjectsContext";
import ServeisContext from "../../../contexts/ServeisContext";

const mapBoxToken = process.env.NEXT_PUBLIC_MAPBOX;

export const ModalServices = (props) => {
    const { show, setshow, title, data, isNew, ...otherProps } = props;
    const { /*serveis,*/ setserveis } = ServeisContext();
    const { objects/*,setObjects*/ } = ObjectsContext();

    const handleClose = () => {
        setshow(false);
    };
    const { user } = useUserContext();

    const url = `https://api.mapbox.com/geocoding/v5/mapbox.places/${data?.lng},${data?.lat}.json?access_token=${mapBoxToken}`;
    // const { data, loading, error } = useFetch({ url });

    const geocoding = () => {
        fetch(url)
            .then((resp) => resp.json())
            .then((d) => {
                if (d) {
                    setValues((prevState) => ({
                        ...prevState,
                        direction: d?.features[0]?.place_name,
                    }));
                }
            })
            .catch((err) => {
                toast.error("Error recuperant l'adreça");
            });
    };

    const [formValidated, setformValidated] = useState(false);

    const handleSubmit = (e) => {
        e.preventDefault();
        e.stopPropagation();
        //Es carreguen al apretar submit per si hi ha
        //canvi de token en el inpass del modal
        const token = user?.token ?? false;
        const urlApi = `/api/data/services`;
        const SendData = async () => {
            fetch(urlApi, {
                method: isNew ? "POST" : "PUT",
                headers: {
                    Accept: "application/json, text/plain, */*",
                    "Content-Type": "application/json",
                    token,
                },
                body: JSON.stringify({ data: formValues }),
            })
                .then((resp) => resp.json())
                .then((d) => {
                    if (!!d.accion) {
                        setserveis((prevState) => {
                            //Si hi ha serveis sino retorna directament                            
                            if (prevState?.length > 0) {
                               
                                const index = prevState.findIndex(
                                    (servei) => servei?._id === d?.data?._id
                                );
                                if (index >= 0) {
                                    prevState[index] = d?.data;
                                } else {
                                    prevState.push(d?.data);
                                }
                                return prevState;
                            }
                            prevState = [];
                            prevState.push(d?.data);
                           
                            return prevState;
                        });
                        setshow(false);
                        toast.info("S'ha actualitzat correctament!");
                    } else {
                        toast.error("No s'ha pogut actualitzar!");
                    }
                })
                .catch((err) => {
                    toast.error("No s'ha pogut actualitzar!");
                });
        };
        const form = e.currentTarget;
        //Comprovar la validesa del formulari
        form.checkValidity() ? SendData() : setformValidated(true);
    };

    const [formValues, handleInputChange, setValues /*,reset*/] = useForm({
        _id: data?._id,
        title: data?.title ?? "",
        lat: data?.geometry?.coordinates[1] ?? data?.lat?.toFixed(6),
        lon: data?.geometry?.coordinates[0] ?? data?.lng?.toFixed(6),
        direction: data?.direction ?? "",
        service: data?.service ?? "",
        result: data?.result ?? "",
        status: data?.status ?? "open",
        objects: data?.objects ?? [],
        dateAssign: data?.dateAssign ?? "",
        dateFinish: data?.dateFinish ?? "",
    });

    const changeMultiSelect = (e) => {
        setValues((prevState) => ({
            ...prevState,
            objects: e,
        }));
        if (formValidated) {
            setformValidated(false);
        }
    };

    const changeInput = (e) => {
        handleInputChange(e);
        if (formValidated) {
            setformValidated(false);
        }
    };

    useEffect(() => {
        isNew && geocoding();
        document.getElementById("titleService").focus();
    }, [url]);

    let options = [];

    if (objects?.data?.length > 0) {
        objects?.data?.forEach((obj) => {           
            const tmp = { value: obj?.object?._id, label: obj?.object?.name };
            options.push(tmp);
        });
    }

    return (
        <>
            <Modal
                id="serviceModal"
                {...otherProps}
                show={show}
                onHide={handleClose}
                backdrop="static"
                animation={false}
                size="lg"
                aria-labelledby="contained-modal-title-vcenter"
                scrollable
            >
                <Modal.Header closeButton>
                    <Modal.Title>{title}</Modal.Title>
                </Modal.Header>
                <Modal.Body>
                    <Form
                        noValidate
                        validated={formValidated}
                        onSubmit={handleSubmit}
                    >
                        <p className="text-center font-weight-bold">
                            Detall servei
                        </p>
                        <div className="mr-4">
                            <GeneralModal
                                changeInput={changeInput}
                                id={"titleService"}
                                label={"Títol"}
                                name={"title"}
                                value={formValues?.title}
                                type={"text"}
                                required
                            />
                            <div style={{ width: "50%" }}>
                                <GeneralModal
                                    changeInput={changeInput}
                                    id={"lat"}
                                    label={"Latitud"}
                                    name={"lat"}
                                    value={formValues?.lat}
                                    type={"number"}
                                    step={"0.000001"}
                                />
                                <GeneralModal
                                    changeInput={changeInput}
                                    id={"lon"}
                                    label={"Longitud"}
                                    name={"lon"}
                                    value={formValues?.lon}
                                    type={"number"}
                                    step={"0.000001"}
                                />
                            </div>
                            <GeneralModal
                                changeInput={changeInput}
                                id={"direction"}
                                label={"Direcció"}
                                name={"direction"}
                                value={formValues?.direction}
                                type={"text"}
                            />
                            <Form.Row className="align-items-center">
                                <Col
                                    xs="auto"
                                    className="my-1 d-flex mt-2"
                                    style={{ flex: "auto" }}
                                >
                                    <Form.Label
                                        className="mt-2"
                                        htmlFor="select1"
                                        style={{ minWidth: "4.7em" }}
                                    >
                                        Patrulla
                                    </Form.Label>
                                    <div style={{ width: "100%" }}>
                                        <Select
                                            placeholder="Vehicles o dispositius"
                                            value={formValues?.objects}
                                            options={options}
                                            isMulti={true}
                                            onChange={changeMultiSelect}
                                        />
                                    </div>
                                    <Form.Control.Feedback type="invalid">
                                        Aquest camp no pot estar en blanc
                                    </Form.Control.Feedback>
                                </Col>
                            </Form.Row>
                            <GeneralModal
                                changeInput={changeInput}
                                id={"service"}
                                label={"Servei"}
                                name={"service"}
                                value={formValues?.service}
                                type={"text"}
                                as={"textarea"}
                            />
                            <GeneralModal
                                changeInput={changeInput}
                                id={"result"}
                                label={"Resultat"}
                                name={"result"}
                                value={formValues?.result}
                                type={"text"}
                                as={"textarea"}
                            />
                        </div>

                        <Form.Row className="align-items-center">
                            <Col xs="auto" className="my-1 d-flex">
                                <Form.Label
                                    className="mt-2"
                                    htmlFor="select1"
                                    style={{ minWidth: "4.7em" }}
                                >
                                    Estat
                                </Form.Label>
                                <Form.Control
                                    as="select"
                                    id="status"
                                    name="status"
                                    style={{ marginRight: "25px" }}
                                    defaultValue={formValues?.status}
                                    onChange={changeInput}
                                    required
                                >
                                    <option checked value="open">
                                        Obert
                                    </option>
                                    <option value="traspas">Traspassat</option>
                                    <option value="closed">Tancat</option>
                                </Form.Control>
                                <Form.Control.Feedback type="invalid">
                                    Aquest camp no pot estar en blanc
                                </Form.Control.Feedback>
                            </Col>
                        </Form.Row>

                        <Form.Row
                            className="d-flex justify-content-center"
                            style={{ marginTop: "20px" }}
                        >
                            <Button type="submit">Desar</Button>
                        </Form.Row>
                    </Form>
                </Modal.Body>                                
            </Modal>
            <style jsx global>{``}</style>
        </>
    );
};
