import Modal from "react-bootstrap/Modal";
import { useRouter } from "next/router";
import { Container, Tab, Tabs } from "react-bootstrap";
import { ModalAccount } from "./Tabs/account/ModalAccount";
import useUserContext from "../../../contexts/UserContext";
import { useFetch } from "../../../Hooks/useFetch";
import Spinner from "../../globals/Spinner";
import { colors } from "../../../styles/theme";
import { ToastContainer } from "react-toastify";
import "react-toastify/dist/ReactToastify.css";
import { ModalSubAccount } from "./Tabs/subAccounts/ModalSubAccount";
import { ModalObjects } from "./Tabs/objects/ModalObjects";
import { ModalMapa } from "./Tabs/map/ModalMapa";

export const ModalSettings = (props) => {
    const { show, setshow, title,showTab ,text, route, ...otherProps } = props;

    const router = useRouter();

    const handleClose = () => {
        setshow(false);
        if (!!route) router.push(route);
    };

    const { user } = useUserContext();

    const token = user?.token ?? false;

    //Data Account
    const urlAccount = `/api/data/account`;
    const {
        data: dataAccount,
        loading: loadingAccount,
        error: errorAccount,
    } = useFetch({ url: urlAccount, token });

    //Data Entity
    const urlEntity = `/api/data/entity`;
    const {
        data: dataEntity,
        loading: loadingEntity,
        error: errorEntity,
    } = useFetch({ url: urlEntity, token });

    return (
        <>
            <Modal
                id="adminManagerModal"
                {...otherProps}
                show={show}
                onHide={handleClose}
                backdrop="static"
                animation={false}
                size="lg"
                aria-labelledby="contained-modal-title-vcenter"
                scrollable
            >
                <Modal.Header closeButton>
                    <Modal.Title>{title}</Modal.Title>
                </Modal.Header>
                <Modal.Body>
                    {!loadingAccount &&
                        !loadingEntity &&
                        !errorAccount &&
                        !errorEntity && (
                            <Container>
                                <Tabs
                                    defaultActiveKey={`${(!!showTab)?showTab:"accountTab"}`}
                                    id="mainTab"
                                    className="mb-3"
                                >
                                    <Tab eventKey="accountTab" title="Compte">
                                        {!loadingAccount && !loadingEntity && (
                                            <ModalAccount
                                                account={dataAccount?.data}
                                                entity={dataEntity?.data}
                                            />
                                        )}
                                    </Tab>
                                    {dataAccount?.data?.admin && (
                                        <Tab
                                            eventKey="subcomptes"
                                            title="Sub-Comptes"
                                        >
                                            {!loadingAccount &&
                                                !loadingEntity && (
                                                    <ModalSubAccount
                                                        account={
                                                            dataAccount.data
                                                        }
                                                    />
                                                )}
                                        </Tab>
                                    )}
                                    {dataAccount?.data?.admin && (
                                        <Tab
                                            eventKey="device"
                                            title="Dispositius"
                                            // disabled
                                        >
                                            {!loadingAccount &&
                                                !loadingEntity && (
                                                    <ModalObjects
                                                        account={
                                                            dataAccount?.data
                                                        }
                                                    />
                                                )}
                                        </Tab>
                                    )}
                                    <Tab
                                        eventKey="map"
                                        title="Mapa"
                                        // disabled
                                    >
                                        {!loadingAccount && (
                                            <ModalMapa
                                                account={dataAccount?.data}
                                            />
                                        )}
                                    </Tab>
                                </Tabs>
                            </Container>
                        )}
                    {(loadingAccount || loadingEntity) && (
                        <div
                            id="spinner"
                            style={{ visibility: "visible" }}
                            className="spinnerDiv"
                        >
                            <Spinner colorspinner={colors.primaryBlue} />
                        </div>
                    )}
                    {(errorAccount || errorEntity) && (
                        <h4>
                            Alguna cosa ha fallat, no es poden carregar les
                            dades, recarregui
                        </h4>
                    )}
                </Modal.Body>
                {/* <Modal.Footer> */}
                {/* <Button onClick={handleClose}>Cerrar</Button> */}
                {/* </Modal.Footer> */}
                <ToastContainer
                    position="top-center"
                    autoClose={2000}
                    closeOnClick
                    draggable
                    pauseOnHover={false}
                />
            </Modal>
            <style jsx global>{`
                #adminManagerModal .modal-content {
                    min-height: 400px;
                }
            `}</style>
        </>
    );
};
