import firebase from "firebase/app";
import { Button } from "../../../../globals/Button";
import { toast } from "react-toastify";
import useUserContext from "../../../../../contexts/UserContext";
import { GeneralModal } from "../../../../formulari/GeneralModal";
import { Form } from "react-bootstrap";
import { useState } from "react";
import { useForm } from "../../../../../Hooks/useForm";

export const ModalDelete = ({ account }) => {
    const { user } = useUserContext();
    

    const [formValidated, setformValidated] = useState(false);

    const initialForm = {
        email: user?.email ?? false,
        password: false,
    };

    const [formValues, handleInputChange /*,reset*/] = useForm(initialForm);

    const changeInput = (e) => {
        handleInputChange(e);
        if (formValidated) {
            setformValidated(false);
        }
    };

    const handleSubmit = (e) => {
        //Es carreguen al apretar submit per si hi ha 
        //canvi de token en el inpass del modal
        const token = user?.token ?? false;
        const url = `/api/data/account`;

        const StartDelete = async () => {
            //return a FirebaseUser or false
            const checkUser = ({ email, password }) => {
                return new Promise((resolve) => {
                    firebase
                        .auth()
                        .signInWithEmailAndPassword(email, password)
                        .then((u) => {
                            //resolve User
                            console.log(u);
                            resolve(u);
                        })
                        .catch((error) => {
                            error.code === "auth/wrong-password"
                                ? toast.error(
                                      "El password introduït es incorrecte"
                                  )
                                : toast.error(
                                      "Error en la identificació de l'usuari"
                                  );
                            resolve(false);
                        });
                });
            };
            
            //Primer comprobar la contrasenya recarregant l'usuari
            const userFirebase = await checkUser(formValues);            

            if (userFirebase) {
                fetch(url, {
                    method: "DELETE",
                    headers: {
                        Accept: "application/json, text/plain, */*",
                        "Content-Type": "application/json",
                        token
                    },
                    body: JSON.stringify({ formValues }),
                })
                    .then((resp) => resp.json())
                    .then((data) => {
                        if (!!data.accion) {      
                            //If Mongo OK Delete user Firebase                      
                            // userFirebase.delete();
                            toast.info("S'ha eliminat el compte correctament!");                           
                        } else {
                            toast.error("No s'ha pogut eliminar el compte!");
                        }
                    })
                    .catch((err) => {
                        toast.error("No s'ha pogut eliminar el compte!");
                    });
            }            
        };

        const form = e.currentTarget;
        e.preventDefault();
        e.stopPropagation();

        //Comprovar la validesa del formulari
        form.checkValidity() ? StartDelete() : setformValidated(true);
    };

    return (
        <>
            <p className="text-center font-weight-bold">Suprimir el compte</p>
            <p className="text-justify">
                Aquesta acció és irreversible, si suprimeix el compte no es
                podrà recuperar i totes les dades relacionades quedaran
                eliminades, també tots els sub-comptes associats
                s&lsquo;eliminaran.
            </p>
            <p>Desitja continuar?</p>
            <Form noValidate validated={formValidated} onSubmit={handleSubmit}>
                <p className="text-center font-weight-bold">
                    Confirmar contrasenya
                </p>
                <GeneralModal
                    changeInput={changeInput}
                    id={"password"}
                    name={"password"}
                    value={formValues.displayName}
                    type={"password"}
                    required
                />
                <Form.Row
                    className="d-flex justify-content-center"
                    style={{ marginTop: "25px" }}
                >
                    <Button type="submit">Eliminar compte</Button>
                </Form.Row>
            </Form>
        </>
    );
};
