/**
 * state --> 0 carrega llista subcomtes
 * state --> 1 edita la subcompte seleccionada
 * state --> 2 formulari crear un subcomte nou
 *
 */

import { useState } from "react";
import { Form, Col } from "react-bootstrap";
import useUserContext from "../../../../../contexts/UserContext";
import { useForm } from "../../../../../Hooks/useForm";
import { GeneralModal } from "../../../../formulari/GeneralModal";
import Image from "next/image";
import MapContext from "../../../../../contexts/MapContext";
import { Button } from "../../../../globals/Button";
import { toast } from "react-toastify";
export const ModalMapa = ({ account }) => {
    const { user } = useUserContext();
    const [formValidated, setformValidated] = useState(false);


    const handleSubmit = (e) => {
        e.preventDefault();
        e.stopPropagation();
        //Es carreguen al apretar submit per si hi ha
        //canvi de token en el inpass del modal
        const token = user?.token ?? false;
        const url = `/api/data/mapconf`;
        const SendData = async () => {
            fetch(url, {
                method: "PUT",
                headers: {
                    Accept: "application/json, text/plain, */*",
                    "Content-Type": "application/json",
                    token,
                },
                body: JSON.stringify({ data: formValues }),
            })
                .then((resp) => resp.json())
                .then((data) => {
                    if (!!data.accion) {
                        toast.info("S'ha actualitzat correctament!");
                    } else {
                        toast.error("No s'ha pogut actualitzar!");
                    }
                })
                .catch((err) => {
                    toast.error("No s'ha pogut actualitzar!");
                });
        };

        const form = e.currentTarget;

        //Comprovar la validesa del formulari
        form.checkValidity() ? SendData() : setformValidated(true);
    };
 
    const [formValues, handleInputChange, setValues /*,reset*/] = useForm({
        lat: account?.map?.geometry?.coordinates[1],
        lon: account?.map?.geometry?.coordinates[0],
        zoom: account?.map?.zoom,
        layer: account?.map?.layer,
        geocoding: account?.map?.geocoding,
        scrollWheelZoom: account?.map?.scrollWheelZoom,
    });

    const changeInput = (e) => {
        handleInputChange(e);
        if (formValidated) {
            setformValidated(false);
        }
    };
    const { map } = MapContext();
    
    const getCoord = () => {
        const coordenades = map?.getCenter();
        setValues((prevState) => ({
            ...prevState,
            lat: coordenades?.lat?.toFixed(6),
            lon: coordenades?.lng?.toFixed(6),
        }));
        document.getElementById("lat").value = coordenades?.lat?.toFixed(6);
        document.getElementById("lon").value = coordenades?.lng?.toFixed(6);
    };

    const getZoom = () => {
        const zoom = map?.getZoom();
        setValues((prevState) => ({
            ...prevState,
            zoom: zoom?.toFixed(0),
        }));
        document.getElementById("zoom").value = zoom?.toFixed(0);
    };

    return (
        <div className="modalMapa">
            <Form noValidate validated={formValidated} onSubmit={handleSubmit}>
                <p className="text-center font-weight-bold">
                    Configuració per defecte del mapa
                </p>
                <Form.Row>
                    <div className="mr-4">
                        <GeneralModal
                            changeInput={changeInput}
                            id={"lat"}
                            label={"Latitud"}
                            name={"lat"}
                            value={formValues?.lat}
                            type={"number"}
                            step={"0.000001"}
                        />
                    </div>
                </Form.Row>
                <Form.Row>
                    <div className="mr-4">
                        <GeneralModal
                            changeInput={changeInput}
                            id={"lon"}
                            label={"Longitud"}
                            name={"lon"}
                            value={formValues?.lon}
                            type={"number"}
                            step={"0.000001"}
                        />
                    </div>
                    <Image
                        src={`/miscelanea/gps.svg`}
                        alt="imatge logo gps"
                        height="30px"
                        width="30px"
                        title="Coordenades actuals del mapa"
                        onClick={getCoord}
                    />
                </Form.Row>
                <Form.Row>
                    <div className="mr-4">
                        <GeneralModal
                            changeInput={changeInput}
                            id={"zoom"}
                            label={"Zoom"}
                            name={"zoom"}
                            value={formValues?.zoom}
                            type={"number"}
                        />
                    </div>
                    <Image
                        src={`/miscelanea/map-search.svg`}
                        alt="imatge mapa"
                        height="30px"
                        width="30px"
                        title="Zoom actual del mapa"
                        onClick={getZoom}
                    />
                </Form.Row>
                <Form.Row className="align-items-center">
                    <Col xs="auto" className="my-1 d-flex">
                        <Form.Label
                            className="mt-2"
                            htmlFor="select1"
                            style={{ minWidth: "4.7em" }}
                        >
                            Layer
                        </Form.Label>
                        <Form.Control
                            as="select"
                            id="layer"
                            name="layer"
                            style={{ marginRight: "25px" }}
                            value={(formValues?.layer)||""}
                            onChange={changeInput}
                            required                            
                        >
                            <option value="">
                                selecciona...
                            </option>
                            <option value="osm">OSM</option>
                            <option value="iccsat">ICC Satèl·lit</option>
                            <option value="icctopo">ICC Topogràfic</option>
                            <option value="osmbw">OSM B/W</option>
                        </Form.Control>
                        <Form.Control.Feedback type="invalid">
                            Aquest camp no pot estar en blanc
                        </Form.Control.Feedback>
                    </Col>
                </Form.Row>
                <GeneralModal
                    changeInput={changeInput}
                    id={"geocoding"}
                    label={"Geocoding"}
                    name={"geocoding"}
                    checked={formValues?.geocoding}
                    type={"checkbox"}
                />
                <GeneralModal
                    changeInput={changeInput}
                    id={"scrollWheelZoom"}
                    label={"Scroll"}
                    name={"scrollWheelZoom"}
                    checked={formValues?.scrollWheelZoom}
                    type={"checkbox"}
                />

                <Form.Row
                    className="d-flex justify-content-center"
                    style={{ marginTop: "25px" }}
                >
                    <Button type="submit">Actualitzar</Button>
                </Form.Row>
            </Form>
        </div>
    );
};
