import React, { useState } from "react";
import Image from "next/image";
import { Col, Form } from "react-bootstrap";
import { GeneralModal } from "../../../../formulari/GeneralModal";
import { Button } from "../../../../globals/Button";
import { useForm } from "../../../../../Hooks/useForm";
import { toast } from "react-toastify";
import useUserContext from "../../../../../contexts/UserContext";

export const ModalObjectsEdit = ({
    stateobj,
    setstateobj,
    objectedit,
    setobjects,
}) => {
    const [formValidated, setformValidated] = useState(false);
    const { user } = useUserContext();

    const handleSubmit = (e) => {
        e.preventDefault();
        e.stopPropagation();
        //Es carreguen al apretar submit per si hi ha
        //canvi de token en el inpass del modal
        const token = user?.token ?? false;
        const url = `/api/data/objects`;
        const SendData = async () => {
            fetch(url, {
                method: stateobj == 1 ? "PUT" : "POST",
                headers: {
                    Accept: "application/json, text/plain, */*",
                    "Content-Type": "application/json",
                    token,
                },
                body: JSON.stringify({ data: formValues }),
            })
                .then((resp) => resp.json())
                .then((data) => {
                    if (!!data.accion) {
                        const object = data?.data;
                        if (stateobj == 2) {
                            setobjects((prevState) => ({
                                objects: [...prevState.objects, object],
                            }));
                        } else {
                            setobjects((prevState) => ({
                                objects: prevState?.objects.map((el) => {
                                    if (el._id === object?._id) {
                                        return {
                                            ...el,
                                            IMEI: object?.IMEI,
                                            marca: object?.marca,
                                            model: object?.model,
                                            name: object?.name,
                                            plateNumber:object?.plateNumber,
                                            group: object?.group,
                                            icon: object?.icon,
                                            GPSdevice: object?.GPSdevice,
                                            visible: object?.visible,
                                            active: object?.active,
                                        };
                                    } else {
                                        return el;
                                    }
                                }),
                            }));
                        }
                        toast.info("S'ha actualitzat correctament!");
                        setstateobj(0);
                    } else {
                        toast.error("No s'ha pogut actualitzar!");
                    }
                })
                .catch((err) => {
                    toast.error("No s'ha pogut actualitzar!");
                });
        };

        const form = e.currentTarget;

        //Comprovar la validesa del formulari
        form.checkValidity() ? SendData() : setformValidated(true);
    };

    const [formValues, handleInputChange /*,reset*/] = useForm({
        _id: objectedit._id,
        idEntity: objectedit.idEntity,
        IMEI: objectedit.IMEI,
        marca: objectedit.marca,
        model: objectedit.model,
        name: objectedit.name,
        VIN: objectedit.VIN,
        plateNumber: objectedit.plateNumber,
        group: objectedit.group,
        icon: objectedit.icon,
        GPSdevice: objectedit.GPSdevice,
        visible: objectedit.visible,
        active: objectedit.active,
        apitoken: objectedit.apitoken,
    });

    const changeInput = (e) => {
        handleInputChange(e);
        if (formValidated) {
            setformValidated(false);
        }
    };

    return (
        <Form noValidate validated={formValidated} onSubmit={handleSubmit}>
            <p className="text-center font-weight-bold">
                Gestió d&lsquo;objectes
            </p>
            <GeneralModal
                changeInput={changeInput}
                id={"IMEI"}
                label={"IMEI"}
                name={"IMEI"}
                value={formValues?.IMEI}
                type={"text"}
                required
            />
            <GeneralModal
                changeInput={changeInput}
                id={"name"}
                label={"Name"}
                name={"name"}
                value={formValues?.name}
                type={"text"}
                required
            />

            <GeneralModal
                changeInput={changeInput}
                id={"marca"}
                label={"Marca"}
                name={"marca"}
                value={formValues?.marca}
                type={"text"}
            />

            <GeneralModal
                changeInput={changeInput}
                id={"model"}
                label={"Model"}
                name={"model"}
                value={formValues?.model}
                type={"text"}
            />            
            {(formValues?.VIN || formValues?.group === "car") && (
                <GeneralModal
                    changeInput={changeInput}
                    id={"VIN"}
                    label={"VIN"}
                    name={"VIN"}
                    value={formValues?.VIN}
                    type={"text"}
                />
            )}
            {(formValues?.plateNumber || formValues?.group === "car") && (
                <GeneralModal
                    changeInput={changeInput}
                    id={"plateNumber"}
                    label={"Matricula"}
                    name={"plateNumber"}
                    value={formValues?.plateNumber}
                    type={"text"}
                />
            )}
            <Form.Row className="align-items-center">
                <Col xs="auto" className="my-1 d-flex">
                    <Form.Label
                        className="mt-2"
                        htmlFor="select1"
                        style={{ minWidth: "4.7em" }}
                    >
                        Grup
                    </Form.Label>
                    <Form.Control
                        as="select"
                        id="group"
                        name="group"
                        style={{ marginRight: "25px" }}
                        value={(formValues?.group)||""}
                        onChange={changeInput}
                        required
                    >
                        <option value="">
                            selecciona...
                        </option>
                        <option value="car">Vehicle</option>
                        <option value="phone">Dispositiu</option>
                    </Form.Control>
                    <Form.Control.Feedback type="invalid">
                        Aquest camp no pot estar en blanc
                    </Form.Control.Feedback>
                </Col>
            </Form.Row>
            <Form.Row className="align-items-center">
                <Col xs="auto" className="my-1 d-flex">
                    <Form.Label
                        className="mt-2"
                        htmlFor="select2"
                        style={{ minWidth: "4.7em" }}
                    >
                        Icona
                    </Form.Label>
                    <Form.Control
                        as="select"
                        id="icon"
                        name="icon"
                        style={{ marginRight: "25px" }}
                        value={(formValues?.icon)||""}                        
                        onChange={changeInput}
                        required
                    >
                        <option value="">
                            selecciona...
                        </option>
                        <option value="car">Cotxe</option>
                        {/* <option value="motorcycle">Motocicleta</option>
                        <option value="phone">Telèfon</option> */}
                        <option value="tablet">Tauleta</option>
                        <option value="van">Furgoneta</option>
                    </Form.Control>
                    <Form.Control.Feedback type="invalid">
                        Aquest camp no pot estar en blanc
                    </Form.Control.Feedback>
                    {formValues?.icon && (
                        <Image
                            src={`/navLateral/slides/vehicles/${formValues.icon}.svg`}
                            alt="imatge del grup"
                            height="50px"
                            width="50px"
                        />
                    )}
                </Col>
            </Form.Row>
            <GeneralModal
                changeInput={changeInput}
                id={"GPSdevice"}
                label={"Dispositiu"}
                name={"GPSdevice"}
                value={formValues?.GPSdevice}
                type={"text"}
            />

            <GeneralModal
                changeInput={changeInput}
                id={"apitoken"}
                label={"apitoken"}
                name={"apitoken"}
                value={formValues?.apitoken}
                type={"text"}
                readOnly
            />
            <GeneralModal
                changeInput={changeInput}
                id={"active"}
                label={"Active"}
                name={"active"}
                checked={formValues?.active}
                type={"checkbox"}
            />
            <GeneralModal
                changeInput={changeInput}
                id={"visible"}
                label={"Visible"}
                name={"visible"}
                checked={formValues?.visible}
                type={"checkbox"}
            />

            <Form.Row
                className="d-flex justify-content-center"
                style={{ marginTop: "25px" }}
            >
                <Button type="submit">
                    {stateobj == 1 ? "Actualitzar" : "Crear"}
                </Button>
                <div className="ml-3">
                    <Button type="button" onClick={() => setstateobj(0)}>Tornar</Button>
                </div>
            </Form.Row>
        </Form>
    );
};
