import React, { useState } from "react";
import Image from "next/image";
import { Button } from "../../../../globals/Button";
import styles from "../../../../../styles/Login.module.css";
import useUserContext from "../../../../../contexts/UserContext";
import { toast } from "react-toastify";
import { Modal } from "react-bootstrap";

export const ModalObjectsTable = ({
    setstateobj,
    setobjectedit,
    setobjects,
    data,
    loading,
}) => {
    const { user } = useUserContext();

    //modal
    const [show, setShow] = useState(false);
    const [account, setaccount] = useState({});
    const [querySearch, setquerySearch] = useState("");

    const handleSearch = (e) => {
        setquerySearch(e.target.value);
    };

    //Sort by name
    // data?.length > 0 && data?.sort((a, b) => a.name - b.name);

    const handleEditClick = (u) => {
        setobjectedit(u);
        setstateobj(1);
    };

    const handleNewClick = () => {
        setobjectedit({});
        setstateobj(2);
    };

    const handleRemoveClick = (u) => {
        setaccount(u);
        setShow(true);
    };

    const confirmRemoveAccount = (object) => {
        const token = user?.token ?? false;
        const url = `/api/data/objects`;
        fetch(url, {
            method: "DELETE",
            headers: {
                Accept: "application/json, text/plain, */*",
                "Content-Type": "application/json",
                token,
            },
            body: JSON.stringify({ data: object }),
        })
            .then((resp) => resp.json())
            .then((d) => {
                if (!!d.accion) {
                    setobjects((prevState) => ({
                        objects: prevState.objects.filter(
                            (item) => item._id !== object._id
                        ),
                    }));
                    toast.info("L'objecte s'ha eliminat correctament!");
                } else {
                    toast.error("No s'ha pogut eliminar!");
                }
            })
            .catch((err) => {
                toast.error("No s'ha pogut eliminar!");
            });
    };

    if (data?.objects?.length > 0) {
        return (
            <>
                <div className="groupSearch">
                    <input
                        id="search"
                        onChange={handleSearch}
                        placeholder="Cerca per IMEI o per Nom"
                        type="search"
                        value={querySearch}
                    />
                </div>
                <div
                    className={`${styles.card} card`}
                    style={{
                        height: "450px",
                        padding: "0%",
                        overflow: "hidden",
                        overflowY: "auto",
                    }}
                >
                    <table>
                        <thead>
                            <tr className="center">
                                <th title="IMEI">IMEI</th>
                                <th title="Name">Nom</th>
                                <th title="Active">Active</th>
                                <th title="Visible">Visible</th>
                                <th title="Accions">Accions</th>
                            </tr>
                        </thead>
                        <tbody>
                            {data?.objects?.map((object) => {
                                return (
                                    (object?.IMEI.includes(querySearch) ||
                                        object?.name
                                            .toUpperCase()
                                            .includes(
                                                querySearch.toUpperCase()
                                            )) && (
                                        <tr
                                            className="center"
                                            key={object?._id}
                                        >
                                            <td
                                                title={object?.IMEI}
                                                style={{ marginTop: "5px" }}
                                            >
                                                {object?.IMEI}
                                            </td>
                                            <td
                                                className="center"
                                                title={object?.name}
                                            >
                                                {object?.name}
                                            </td>
                                            <td style={{ marginTop: "3px" }}>
                                                <Image
                                                    src={
                                                        object?.active
                                                            ? `/miscelanea/check.svg`
                                                            : `/miscelanea/aspa.svg`
                                                    }
                                                    alt="Picture of the author"
                                                    width={15}
                                                    height={15}
                                                    title={
                                                        object?.active
                                                            ? "Cert"
                                                            : "Fals"
                                                    }
                                                />
                                            </td>
                                            <td style={{ marginTop: "3px" }}>
                                                <Image
                                                    src={
                                                        object?.visible
                                                            ? `/miscelanea/check.svg`
                                                            : `/miscelanea/aspa.svg`
                                                    }
                                                    alt="Picture of the author"
                                                    width={15}
                                                    height={15}
                                                    title={
                                                        object?.visible
                                                            ? "Cert"
                                                            : "Fals"
                                                    }
                                                />
                                            </td>
                                            <td style={{ marginTop: "3px" }}>
                                                <div className="gridAccions">
                                                    <Image
                                                        src="/miscelanea/lapiz.svg"
                                                        alt="Icona de llapis"
                                                        width={20}
                                                        height={20}
                                                        title="Editar"
                                                        onClick={() =>
                                                            handleEditClick(
                                                                object
                                                            )
                                                        }
                                                    />
                                                    <Image
                                                        src="/miscelanea/paperera.svg"
                                                        alt="Icona de llapis"
                                                        width={20}
                                                        height={20}
                                                        title="Esborrar"
                                                        cursor="pointer"
                                                        onClick={() => {
                                                            handleRemoveClick(
                                                                object
                                                            );
                                                        }}
                                                    />
                                                </div>
                                            </td>
                                        </tr>
                                    )
                                );
                            })}
                        </tbody>
                    </table>
                </div>
                <div
                    className="d-flex justify-content-center"
                    style={{ marginTop: "25px" }}
                >
                    <Button onClick={handleNewClick}>Afegir nova</Button>
                </div>
                <Modal
                    show={show}
                    onHide={() => setShow(false)}
                    backdrop="static"
                    animation={false}
                    size="lg"
                    aria-labelledby="contained-modal-title-vcenter"
                    scrollable
                >
                    <Modal.Header closeButton>
                        <Modal.Title>Suprimir objecte</Modal.Title>
                    </Modal.Header>
                    <Modal.Body>
                        Esteu segur que voleu suprimir l&lsquo;objecte?
                    </Modal.Body>
                    <Modal.Footer>
                        <Button
                            variant="primary"
                            onClick={() => {
                                confirmRemoveAccount(account);
                                setShow(false);
                            }}
                        >
                            Suprimir
                        </Button>
                        <Button
                            variant="secondary"
                            onClick={() => setShow(false)}
                        >
                            Cancelar
                        </Button>
                    </Modal.Footer>
                </Modal>
                <style jsx>{`
                    .groupSearch {
                        display: flex;
                    }
                    #search {
                        background-size: 15px 15px;
                        background: #fff url(navLateral/slides/search.svg) 10px
                            center no-repeat;
                        border: 1px solid var(--greyBorder);
                        height: 28px;
                        text-indent: 30px;
                        width: 90%;
                        margin-left: 15px;
                    }
                    #search:focus {
                        border: 1px solid var(--primaryBlue);
                        outline: none;
                    }

                    .subAccount {
                        margin: 0px;
                    }
                    .center {
                        align-self: center;
                        text-align: center;
                    }
                    table {
                        border-radius: 10px;
                        background: #007bff3e;
                        margin-top: 1.2 rem;
                        width: 100%;
                        margin-bottom: 2 rem;
                        font-size: small;
                    }

                    tbody > tr > td,
                    thead > tr > th {
                        padding: 5px;
                        text-overflow: ellipsis;
                        overflow: hidden;
                        overflow-wrap: normal;
                    }
                    .gridAccions {
                        display: grid;
                        grid-template-columns: 50% 50%;
                        cursor: pointer;
                    }

                    thead > tr {
                        padding: 5px;
                        width: 100%;
                        display: grid;
                        grid-template-columns: 30% 17% 15% 15% 23%;
                        margin-top: 10 px;
                    }
                    thead {
                        display: flex;
                        height: 2.5rem;
                    }
                    tbody > :global(tr: first-child) {
                        border-top: 1px #8a8a8a90 solid;
                    }
                    tbody > :global(tr) {
                        border-bottom: 1px #8a8a8a90 solid;
                        height: 2.5rem;
                        display: grid;
                        grid-template-columns: 30% 17% 15% 15% 23%;
                    }
                    tbody > :global(tr:nth-last-child(-n + 1)) {
                        border-radius: 10px;
                    }
                    @media (max-width: 600px) {
                        .card {
                            margin: 10px -10px;
                        }
                    }
                `}</style>
            </>
        );
    } else {
        return (
            <div
                className="d-flex justify-content-center"
                style={{ marginTop: "25px" }}
            >
                <Button onClick={handleNewClick}>Afegir nova</Button>
            </div>
        );
    }
};
