import Modal from "react-bootstrap/Modal";

import { Col, Form } from "react-bootstrap";
import { toast } from "react-toastify";

import useUserContext from "../../../contexts/UserContext";
import "react-toastify/dist/ReactToastify.css";
import { GeneralModal } from "../../formulari/GeneralModal";
import { Button } from "../../globals/Button";
import { useEffect, useState } from "react";
import { useForm } from "../../../Hooks/useForm";
import { useMap } from "react-leaflet";
import ZonesContext from "../../../contexts/ZonesContext";

export const ModalZona = (props) => {
    const { show, setshow, title, mark, setmark, isNew, ...otherProps } = props;
    const map = useMap();
    const handleClose = () => {
        setshow(false);
    };

    const { user } = useUserContext();
    const { zones, setzones } = ZonesContext();

    const [formValidated, setformValidated] = useState(false);

    
    const handleSubmit = (e) => {
        e.preventDefault();
        e.stopPropagation();
        //Es carreguen al apretar submit per si hi ha
        //canvi de token en el inpass del modal
        const token = user?.token ?? false;
        const url = `/api/data/zones`;
        const SendData = async () => {
            fetch(url, {
                method: isNew ? "POST" : "PUT",
                headers: {
                    Accept: "application/json, text/plain, */*",
                    "Content-Type": "application/json",
                    token,
                },
                body: JSON.stringify({ data: formValues }),
            })
                .then((resp) => resp.json())
                .then((d) => {
                    if(d?.data?.type=="marker"){
                        setzones((prevState) => {                              
                            prevState.main.marks.push(d?.data);                       
                            return prevState;
                        });

                    }else{                        
                        setzones((prevState) => {                              
                            prevState.main.polygons.push(d?.data);                       
                            return prevState;
                        });
                    }

                    map.removeLayer(mark);
                    toast.info("S'ha actualitzat correctament!");
                    setshow(false);
                })
                .catch((err) => {
                    console.log(err)
                    toast.error("No s'ha pogut actualitzar!");
                });
        };
        const form = e.currentTarget;
        //Comprovar la validesa del formulari
        form.checkValidity() ? SendData() : setformValidated(true);
    };

    const GetLatsLongs = () => {
        let tmp = [];
        try {
            if (["polygon", "rectangle"].includes(mark?.layerType)) {
                if (mark?.getLatLngs().length > 0) {
                    mark.getLatLngs()[0].map((m) => {
                        tmp.push([m?.lat, m?.lng]);
                    });
                }
            } else if (mark?.layerType === "polyline") {
                if (mark?.getLatLngs().length > 0) {
                    mark.getLatLngs().map((m) => {
                        tmp.push([m?.lat, m?.lng]);
                    });
                }
            } else if (["circle", "marker"].includes(mark?.layerType)) {
                tmp.push([mark.getLatLng().lat, mark.getLatLng().lng]);
            }

            return tmp;
        } catch (error) {
            console.log(error);
            return tmp;
        }
    };
    let coordinates = GetLatsLongs();

    const [formValues, handleInputChange, setValues /*,reset*/] = useForm({
        name: "",
        description: "",
        group: "",
        color: "#1343de",
        visible: true,
        type: mark?.layerType,       
        coordinates: coordinates,
        radius: mark?.layerType === "circle" ? mark.getRadius() : 0,
    });

    const changeInput = (e) => {
        handleInputChange(e);
        if (formValidated) {
            setformValidated(false);
        }
    };

    useEffect(() => {
        document.getElementById("nameZone").focus();
    }, []);

    return (
        <>
            <Modal
                id="modalZona"
                {...otherProps}
                show={show}
                onHide={handleClose}
                backdrop="static"
                animation={false}
                size="lg"
                aria-labelledby="contained-modal-title-vcenter"
                scrollable
            >
                <Modal.Header closeButton>
                    <Modal.Title>{title}</Modal.Title>
                </Modal.Header>
                <Modal.Body>
                    <Form
                        noValidate
                        validated={formValidated}
                        onSubmit={handleSubmit}
                    >
                        <p className="text-center font-weight-bold">
                            Detall Zones / Marques
                        </p>
                        <div className="mr-4">
                            <GeneralModal
                                changeInput={changeInput}
                                id={"nameZone"}
                                label={"Nom"}
                                name={"name"}
                                value={formValues?.name}
                                type={"text"}
                                required
                            />
                            <GeneralModal
                                changeInput={changeInput}
                                id={"description"}
                                label={"Descripció"}
                                name={"description"}
                                value={formValues?.description}
                                type={"text"}
                            />
                            <div style={{ width: "30%" }}>
                                <GeneralModal
                                    changeInput={changeInput}
                                    id={"color"}
                                    label={"Color"}
                                    name={"color"}
                                    value={formValues?.color}
                                    type={"color"}
                                />
                            </div>
                            <Form.Row className="align-items-center">
                                <Col xs="auto" className="my-1 d-flex">
                                    <Form.Label
                                        className="mt-2"
                                        htmlFor="select2"
                                        style={{ minWidth: "4.7em" }}
                                    >
                                        Grup
                                    </Form.Label>
                                    <Form.Control
                                        as="select"
                                        id="group"
                                        name="group"
                                        style={{ marginRight: "25px" }}
                                        value={formValues?.group || ""}
                                        onChange={changeInput}
                                        required
                                    >
                                        <option value="">selecciona...</option>
                                        <option value="zona">Zona</option>
                                        <option value="districte">
                                            Districte
                                        </option>
                                        <option value="barri">Barri</option>
                                        <option value="city">Municipi</option>
                                        <option value="altres">Altres</option>
                                    </Form.Control>
                                    <Form.Control.Feedback type="invalid">
                                        Aquest camp no pot estar en blanc
                                    </Form.Control.Feedback>
                                </Col>
                            </Form.Row>
                            <GeneralModal
                                changeInput={changeInput}
                                id={"visible"}
                                label={"Visible"}
                                name={"visible"}
                                checked={formValues?.visible}
                                type={"checkbox"}
                            />
                        </div>
                        <Form.Row
                            className="d-flex justify-content-center"
                            style={{ marginTop: "20px" }}
                        >
                            <Button type="submit">Desar</Button>
                        </Form.Row>
                    </Form>
                </Modal.Body>
            </Modal>
            <style jsx global>{``}</style>
        </>
    );
};
