import React, { useEffect } from "react";
import NavLateral from "./MenuLateral/NavLateral";
import dynamic from "next/dynamic";
import TopPanel from "./TopPanel/TopPanel";
import ObjectsContext from "../../contexts/ObjectsContext";
import { useFetchData } from "../../Hooks/useFetchData";
import { useFetch } from "../../Hooks/useFetch";
import useUserContext from "../../contexts/UserContext";
import ServeisContext from "../../contexts/ServeisContext";
import { usePusher } from "../../Hooks/usePusher";
import { ToastContainer } from "react-toastify";
import AccountContext from "../../contexts/AccountContext";
import ZonesContext from "../../contexts/ZonesContext";

//Carrega el mapa sense el SSR de NextJS
const MapWithNoSSR = dynamic(() => import("../../components/main/Mapa/Map"), {
    ssr: false,
});

const Main = () => {
    const { user } = useUserContext();
    const { token } = user ?? false;

    //Contexts
    const { setObjects } = ObjectsContext();
    const { serveis, setserveis } = ServeisContext();
    const { /*account,*/ setAccount } = AccountContext();
    const { /*zone,*/ setzones } = ZonesContext();

    //Get services
    const urlServices = `/api/data/services`;
    const {
        data: dataServices,
        loading: loadingServices,
        error: errorServices,
    } = useFetch({ url: urlServices, token });

    //Get MainData
    const { loading, data, error } = useFetchData(
        { url: `api/data/mainData` },
        { token }
    );
       
    //Data Account
    const urlAccount = `/api/data/account`;
    const {
        data: dataAccount,
        loading: loadingAccount,
        error: errorAccount,
    } = useFetch({ url: urlAccount, token });

    //Get Zones
    const urlZones = `/api/data/zones`;
    const {
        data: dataZones,
        loading: loadingZones,
        error: errorZones,
    } = useFetch({ url: urlZones, token });

    useEffect(() => {
        setserveis(dataServices?.data);
    }, [dataServices, setserveis]);

    useEffect(() => {
        setAccount(dataAccount?.data);
    }, [dataAccount, setAccount]);

    useEffect(() => {
        setzones(dataZones?.data);
    }, [dataZones, setzones]);

    useEffect(() => {
        !loading &&
            setObjects((prevState) => {
                data?.length > 0 &&
                    data.map((currentEl) => {
                        //Mes de 30 minuts sense senyal, posar ignition directament a false
                        const timestamp = new Date(
                            currentEl.properties.timestamp
                        );
                        const currentDate = new Date();
                        const diffMinutes = Math.round(
                            (currentDate - timestamp) / (60 * 1000),
                            0
                        );

                        //Si més de 30 minuts sense enviar, posar la unitat en parat.
                        // diffMinutes > 30 &&
                        //     (currentEl.properties.ioElements.ignition = false);

                        //Conserva el visible si esta activat o desactivat
                        prevState?.data
                            ? prevState.data.map((prevEl) => {
                                  currentEl.object._id === prevEl.object._id &&
                                      (currentEl.object.visible =
                                          prevEl.object.visible);
                              })
                            : //Si més de 12 hores que no mostri de manera default una unitat
                              diffMinutes > 720 &&
                              (currentEl.object.visible = false);

                        //Assigna un camp services amb els serveis que te o pot tenir
                        //assignat un vehicle concret
                        currentEl.services = [];
                        if (serveis?.length > 0) {
                            const serveisFil = serveis.filter((product) =>
                                product.objects.some((item) => {
                                    if (product?.status === "closed") {
                                        return false;
                                    }
                                    return item.value === currentEl.object._id;
                                })
                            );
                            !!serveisFil && (currentEl.services = serveisFil);
                        }
                    });
                return {
                    loading: loading,
                    error: error,
                    data: data,
                };
            });
    }, [loading]);

    const idEntity = dataAccount?.data?.idEntity ?? "";
    const { dataPusher, loadingPusher, errorPusher } = usePusher(
        { idEntity },
        `services`
    );
    //PUSHERS SERVICES
    useEffect(() => {
        //Si Hi ha algun missatge a modificar i no es d'un mateix
        if (dataPusher && dataPusher?.emmit !== token) {
            //Modifica l'estat dels missatges
            setserveis((prevState) => {
                //Si hi ha serveis sino retorna directament
                if (prevState?.length > 0) {
                    const index = prevState.findIndex(
                        (servei) => servei?._id === dataPusher?.service?._id
                    );
                    if (index >= 0) {
                        prevState[index] = dataPusher?.service;
                    } else {
                        prevState.push(dataPusher?.service);
                    }
                    return prevState;
                }
                prevState = [];
                prevState.push(dataPusher?.service);
                return prevState;
            });
        }
    }, [dataPusher, setserveis]);

    
    return (
        <div className="app">
            <div className="containerApp">
                <NavLateral />
                <section className="sectioncontainer">
                    <TopPanel margin={5} data={dataAccount?.data} />
                    <MapWithNoSSR
                        data={dataAccount?.data}
                        loading={loadingAccount}
                        error={errorAccount}
                    />
                    <ToastContainer
                        position="top-center"
                        autoClose={2000}
                        closeOnClick
                        draggable
                        pauseOnHover={false}
                    />
                </section>
            </div>

            <style jsx>{`
                .sectioncontainer {
                    margin-left: 5rem;
                }
                /*Landscape & Portrait*/
                @media (max-width: 600px), (max-height: 600px) {
                    .sectioncontainer {
                        margin-left: 0rem;
                    }
                }
            `}</style>
        </div>
    );
};

export default Main;
