import { useState } from "react";
import { FeatureGroup, LayersControl, TileLayer } from "react-leaflet";
import { EditControl } from "react-leaflet-draw";
import ZonesContext from "../../../contexts/ZonesContext";

import { ModalAmpliada } from "../../customModals/ModalAmpliada/ModalAmpliada";
import Markers from "./Markers";
import Serveis from "./Serveis";
import Zones from "./Zones";

const Layers = () => {
    const [ampliades, setampliades] = useState(false);
    const [unitat, setunitat] = useState({});

    const { zones, setzones } = ZonesContext();

    return (
        <>
            <LayersControl name="principal" position="topright">
                <LayersControl.BaseLayer checked name="OSM">
                    <TileLayer
                        maxZoom="19"
                        attribution='&copy; <a href="http://osm.org/copyright">OpenStreetMap</a> contributors'
                        url="https://tile.openstreetmap.org/{z}/{x}/{y}.png"
                    />
                </LayersControl.BaseLayer>
                <LayersControl.BaseLayer name="ICC Satèl·lit">
                    <TileLayer
                        maxZoom="19"
                        attribution="&copy; Institut Cartogràfic i Geològic de Catalunya"
                        url="https://geoserveis.icgc.cat/icc_mapesmultibase/noutm/wmts/orto/GRID3857/{z}/{x}/{y}.jpeg"
                    />
                </LayersControl.BaseLayer>
                <LayersControl.BaseLayer name="ICC Topogràfic">
                    <TileLayer
                        maxZoom="19"
                        attribution="&copy; Institut Cartogràfic i Geològic de Catalunya"
                        url="https://geoserveis.icgc.cat/icc_mapesmultibase/noutm/wmts/topo/GRID3857/{z}/{x}/{y}.jpeg"
                    />
                </LayersControl.BaseLayer>
                {/* <LayersControl.BaseLayer name="Satèl·lit MapBox">
                    <TileLayer
                        url={`https://api.mapbox.com/styles/v1/rulomo/ckw6abmox1hvb14mve5cjj76r/tiles/256/{z}/{x}/{y}@2x?access_token=pk.eyJ1IjoicnVsb21vIiwiYSI6ImNrcmRiZXUxNDAxOWEydmxoandzYmZkZ2gifQ.XGWYlu4wm1G17hzPCXM3kQ`}
                        attribution='Map data &copy; <a href="https://www.openstreetmap.org/">OpenStreetMap</a> contributors, <a href="https://creativecommons.org/licenses/by-sa/2.0/">CC-BY-SA</a>, Imagery &copy; <a href="https://www.mapbox.com/">Mapbox</a>'
                    />
                </LayersControl.BaseLayer> */}
                <LayersControl.BaseLayer name="OSM Black And White">
                    <TileLayer
                        maxZoom="19"
                        attribution='&copy; <a href="http://osm.org/copyright">OpenStreetMap</a> contributors'
                        url="https://tiles.wmflabs.org/bw-mapnik/{z}/{x}/{y}.png"
                    />
                </LayersControl.BaseLayer>
                <LayersControl.Overlay checked name="Vehicles">
                    <FeatureGroup>
                        <Markers
                            setampliades={setampliades}
                            setunitat={setunitat}
                        />
                    </FeatureGroup>
                </LayersControl.Overlay>
                <LayersControl.Overlay checked name="Serveis">
                    <FeatureGroup>
                        <Serveis />
                    </FeatureGroup>
                </LayersControl.Overlay>
                <LayersControl.Overlay checked name="Zones">
                    <FeatureGroup>
                        {zones && <Zones zones={zones} />}
                    </FeatureGroup>
                </LayersControl.Overlay>
                <LayersControl.Overlay checked name="Marques">
                    <FeatureGroup>{/* <Marks /> */}</FeatureGroup>
                </LayersControl.Overlay>
            </LayersControl>
            {
                <ModalAmpliada
                    setshow={setampliades}
                    show={ampliades}
                    title={"Dades ampliades"}
                    data={unitat}
                />
            }
        </>
    );
};

export default Layers;

// OpenStreetMap Mapnick:  http://tile.openstreetmap.org/{z}/{x}/{y}.png
// OSM Black and White: http://tiles.wmflabs.org/bw-mapnik/{z}/{x}/{y}.png
// ICC:  https://geoserveis.icgc.cat/icc_mapesmultibase/noutm/wmts/topo/GRID3857/{z}/{x}/{y}.jpeg
// Google Maps: https://mt1.google.com/vt/lyrs=r&x={x}&y={y}&z={z}
// Google Satellite: http://www.google.cn/maps/vt?lyrs=s@189&gl=cn&x={x}&y={y}&z={z}
// Google Hybrid: https://mt1.google.com/vt/lyrs=y&x={x}&y={y}&z={z}
// Google Terrain: https://mt1.google.com/vt/lyrs=t&x={x}&y={y}&z={z}
// Google Traffic: https://mt1.google.com/vt?lyrs=h@159000000,traffic|seconds_into_week:-1&style=3&x={x}&y={y}&z={z}
// ESRI Imagery/Satellite: https://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/{z}/{y}/{x}
// ESRI National Geographic: http://services.arcgisonline.com/ArcGIS/rest/services/NatGeo_World_Map/MapServer/tile/{z}/{y}/{x}
// ESRI Physical: https://server.arcgisonline.com/ArcGIS/rest/services/World_Physical_Map/MapServer/tile/{z}/{y}/{x}
// ESRI Streets: https://server.arcgisonline.com/ArcGIS/rest/services/World_Street_Map/MapServer/tile/{z}/{y}/{x}
// ESRI Terrain: https://server.arcgisonline.com/ArcGIS/rest/services/World_Terrain_Base/MapServer/tile/{z}/{y}/{x}
// ESRI Topo: https://server.arcgisonline.com/ArcGIS/rest/services/World_Topo_Map/MapServer/tile/{z}/{y}/{x}
// ESRI Transportation: https://server.arcgisonline.com/ArcGIS/rest/services/Reference/World_Transportation/MapServer/tile/{z}/{y}/{x}
