import React, { useState } from "react";
import {
    FeatureGroup,
    LayersControl,
    MapContainer,
    useMap,
} from "react-leaflet";
import "leaflet/dist/leaflet.css";
import "leaflet-defaulticon-compatibility/dist/leaflet-defaulticon-compatibility.css";
import "leaflet-defaulticon-compatibility";
import "leaflet-contextmenu";
import "leaflet-contextmenu/dist/leaflet.contextmenu.css";
import SearchField from "./SearchField";
import Layers from "./Layers";
import Control from "react-leaflet-custom-control";

import Spinner from "../../../components/globals/Spinner";

import { colors } from "../../../styles/theme";
import styles from "../../../styles/Login.module.css";

import MapContext from "../../../contexts/MapContext";
import Maplibs from "./Maplibs";
import slideMenuContext from "../../../contexts/SlideMenuContext";

import "leaflet-draw/dist/leaflet.draw.css";
import DrawControl from "./Draw";
import { useRef } from "react";

const Map = ({ data: account, loading, error }) => {
    //Posem el mapa en context per a poder utilitzarlo a qualsevol lloc
    const { map, setmap } = MapContext();
    const { action } = slideMenuContext();
    const [showDraw, setshowDraw] = useState(true);

    let zoom = account?.map?.zoom;
    let coordenades;
    if (account?.map?.geometry?.coordinates?.length) {
        coordenades = [
            account?.map?.geometry?.coordinates[1],
            account?.map?.geometry?.coordinates[0],
        ];
    }

    const fullScreen = (e) => {
        // const app = document.getElementsByClassName("App")[0];
        let elem = document.body;
        e.target.classList.toggle("expand");
        if (document.fullscreenElement) {
            document.exitFullscreen();
        } else if (elem.requestFullscreen) {
            elem.requestFullscreen();
        } else if (elem.msRequestFullscreen) {
            elem.msRequestFullscreen();
        } else if (elem.mozRequestFullScreen) {
            elem.mozRequestFullScreen();
        } else if (elem.webkitRequestFullscreen) {
            elem.webkitRequestFullscreen();
        }
    };

    const returnCenter = (e) => {
        map.flyTo(coordenades, zoom);
    };

    const handleShowDraw = (setshowDraw) => {
        let draw = document.querySelectorAll(
            ".leaflet-bottom.leaflet-left > .leaflet-draw.leaflet-control"
        );
        if (showDraw) {
            draw[0].style.visibility = "visible";
        } else {
            draw[0].style.visibility = "hidden";
        }
        setshowDraw((prevState) => {
            return !prevState;
        });
    };

    return (
        <>
            {!loading && !error && (
                <div
                    className={`containerMap ${
                        action === "toggle" ? "toggleOpen" : "toggleClosed"
                    }`}
                >
                    <MapContainer
                        id="map"
                        center={coordenades ? coordenades : [41.7938, 1.7962]}
                        zoom={account?.map?.zoom || 8}
                        scrollWheelZoom={!!account?.map?.scrollWheelZoom}
                        style={{ height: "100%" }}
                        contextmenu={true}
                        whenCreated={(mapa) => {
                            setmap(mapa);
                        }}
                    >
                        {(account?.map?.geocoding ||
                            account?.map?.geocoding === "undefined") && (
                            <SearchField />
                        )}

                        <Layers />
                        <Maplibs />

                        <DrawControl />

                        <Control position="topleft">
                            <a
                                id="expandControl"
                                title="Pantalla completa"
                                onClick={fullScreen}
                            />
                            <a
                                id="returnCenter"
                                title="Centrar mapa"
                                onClick={returnCenter}
                            />
                        </Control>
                        <Control position="bottomleft">
                            <a
                                id="expandDraw"
                                className={`${
                                    showDraw ? `openDraw` : `closedDraw`
                                }`}
                                title="Mostrar Eines Dibuix"
                                onClick={() => handleShowDraw(setshowDraw)}
                            />
                        </Control>
                    </MapContainer>
                    <style jsx>{`
                        :global(.leaflet-bottom > .leaflet-draw) {
                            visibility: hidden;
                        }
                        .containerMap {
                            height: 95%;
                            position: fixed;
                            margin-top: 3rem;
                            width: calc(100vw - 5rem);
                            z-index: 0;
                        }
                        /*.containerMap.toggleOpen {
                            margin-left: 22rem;
                            width: calc(100vw - 27rem);
                            -webkit-transition-delay: 600ms;
                            -moz-transition-delay: 600ms;
                            -o-transition-delay: 600ms;
                            transition-delay: 600ms;
                        }
                        .containerMap.toogleClosed {
                            margin-left: 22rem;
                            width: calc(100vw - 5rem);
                            -webkit-transition-delay: 50ms;
                            -moz-transition-delay: 50ms;
                            -o-transition-delay: 50ms;
                            transition-delay: 50ms;
                        }*/
                        :global(.toggleOpen .leaflet-bottom.leaflet-left),
                        :global(.toggleOpen .leaflet-top.leaflet-left) {
                            margin-left: 22rem;
                            -webkit-transition-delay: 500ms;
                            -moz-transition-delay: 500ms;
                            -o-transition-delay: 500ms;
                            transition-delay: 500ms;
                        }
                        :global(.leaflet-contextmenu) {
                            text-align: center;
                        }
                        :global(.leaflet-contextmenu-item) {
                            cursor: pointer;
                        }
                        #expandDraw {
                            background-image: url(/miscelanea/expansion.png);
                            background-repeat: no-repeat;
                            background-size: 20px 20px;
                            -webkit-background-clip: padding-box;
                            background-clip: padding-box;
                        }

                        .openDraw {
                            transform: rotate(180deg);
                        }
                        .closedDraw {
                            transform: rotate(0deg);
                        }

                        #expandControl {
                            background: #fff url("/miscelanea/fullscreen.png")
                                no-repeat 0 0;
                            cursor: pointer;
                        }
                        #returnCenter {
                            background: #fff url("/miscelanea/center.svg")
                                no-repeat 0 0;
                            cursor: pointer;
                        }
                        #expandControl:not(.expand) {
                            background-position: 2px 2px !important;
                        }
                        .expand {
                            background-position: 2px -24px !important;
                        }
                        /*Landscape & Portrait*/
                        @media (max-width: 600px), (max-height: 600px) {
                            .containerMap {
                                margin-top: 3.75rem;
                                width: calc(100vw);
                                height: 93.5%;
                            }
                        }
                    `}</style>
                </div>
            )}
            {loading && (
                <div
                    id="spinner"
                    style={{ visibility: "hidden" }}
                    className={styles.spinnerDiv}
                >
                    <Spinner colorspinner={colors.primaryBlue} />
                </div>
            )}
        </>
    );
};

export default Map;
