import ObjectsContext from "../../../contexts/ObjectsContext";
import { Marker, Popup, Tooltip } from "react-leaflet";
import * as pointers from "./pointerIcon";
import Link from "next/link";

import { convertDate } from "../../../utils/lib/dates";

const Markers = ({ setampliades, setunitat }) => {
    const {
        /*objects,*/ setObjects,
        objects: { data },
    } = ObjectsContext();

    const handleChangeChk = ({ target, unitat }) => {
        setObjects((prevState) => ({
            error: prevState.error,
            loading: prevState.loading,
            data: prevState.data.map((el) => {
                if (el.object._id === unitat.object._id) {
                    return {
                        ...el,
                        object: { ...el.object, visible: false },
                    };
                } else {
                    return el;
                }
            }),
        }));
    };

    const handleAmpliadesclick = (u) => {
        setampliades(true);
        setunitat(u);
    };

    return (
        <>
            {data?.map((unitat) => {
                const { geometry, properties, object } = unitat;
                const { coordinates } = geometry;
                let colorIcon = `${
                    unitat?.services?.length ? "Blue" : "Black"
                }`;

                if (object.visible) {
                    return (
                        <Marker
                            icon={pointers[`${object?.icon}${colorIcon}`]}
                            id={object?._id}
                            key={object?._id}
                            position={[coordinates[1], coordinates[0]]}
                            draggable={false}
                            animate={false}
                            object={object}
                        >
                            {colorIcon === "Blue" && (
                                <Tooltip
                                    className={`tipVisible Blue`}
                                    direction="top"
                                    opacity={1}
                                    permanent
                                >
                                    {object?.name}
                                </Tooltip>
                            )}
                            {colorIcon === "Black" && (
                                <Tooltip
                                    className={`tipVisible Black`}
                                    direction="top"
                                    opacity={1}
                                    permanent
                                >
                                    {object?.name}
                                </Tooltip>
                            )}
                            <Popup>
                                <table>
                                    <tbody>
                                        <tr>
                                            <td>
                                                <strong>Nom:</strong>
                                            </td>
                                            <td>
                                                {object?.name.toUpperCase()}
                                            </td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <strong>Tipus:</strong>
                                            </td>
                                            <td>{object?.group}</td>
                                        </tr>
                                        {/* <tr>
                                            <td>
                                                <strong>Address:</strong>
                                            </td>
                                            <td>E30, 14778, Germany</td>
                                        </tr>{" "} */}
                                        <tr>
                                            <td>
                                                <strong>Posició:</strong>
                                            </td>
                                            <td>
                                                <Link
                                                    href={`https://maps.google.com/maps?q=${coordinates[1]},${coordinates[0]}&amp;t=m`}
                                                >
                                                    <a
                                                        target="_blank"
                                                        rel="noreferrer"
                                                    >
                                                        {coordinates[1].toFixed(
                                                            5
                                                        )}
                                                        °,
                                                        {coordinates[0].toFixed(
                                                            5
                                                        )}
                                                        °
                                                    </a>
                                                </Link>
                                            </td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <strong>Altitud:</strong>
                                            </td>
                                            <td>
                                                {properties?.gps?.altitude} m
                                            </td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <strong>Angle:</strong>
                                            </td>
                                            <td>{properties?.gps?.angle}°</td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <strong>Velocitat:</strong>
                                            </td>
                                            <td>
                                                {properties?.gps?.speed} km/h
                                            </td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <strong>Data:</strong>
                                            </td>
                                            <td>
                                                {convertDate(
                                                    properties?.timestamp
                                                )}
                                            </td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <strong>Visible:</strong>
                                            </td>
                                            <td>
                                                <input
                                                    checked={object?.visible}
                                                    title="Veure"
                                                    className="pointer"
                                                    type="checkbox"
                                                    style={{
                                                        textAlign: "center",
                                                    }}
                                                    onChange={(e) =>
                                                        handleChangeChk({
                                                            target: e.target,
                                                            unitat,
                                                        })
                                                    }
                                                />
                                            </td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <Link
                                                    href={`https://www.google.com/maps?q&layer=c&cbll=${coordinates[1]},${coordinates[0]}`}
                                                >
                                                    <a
                                                        target="_blank"
                                                        rel="noreferrer"
                                                    >
                                                        {"Street View"}
                                                    </a>
                                                </Link>
                                            </td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <a
                                                    style={{
                                                        cursor: "pointer",
                                                    }}
                                                    rel="noreferrer"
                                                    onClick={() =>
                                                        handleAmpliadesclick(
                                                            unitat
                                                        )
                                                    }
                                                >
                                                    {"Ampliades"}
                                                </a>
                                            </td>
                                        </tr>
                                    </tbody>
                                </table>
                            </Popup>
                        </Marker>
                    );
                }
            })}
            <style jsx global>{`
                .tipVisible {
                    border: none !important;
                    box-shadow: none !important;
                    background-color: transparent !important;
                    font-weight: 800;
                    font-size: smaller;
                }
                .tipVisible.Black {
                    color: black;
                }
                .tipVisible.Blue {
                    color: blue;
                }
                .leaflet-tooltip-top:before {
                    border-top-color: transparent !important;
                }
            `}</style>
        </>
    );
};

export default Markers;
