import { useMap } from "react-leaflet";
import "leaflet-geosearch/dist/geosearch.css";
import { GeoSearchControl, MapBoxProvider } from "leaflet-geosearch";
import { useEffect } from "react";
import * as pointers from "../Mapa/pointerIcon";
import useUserContext from "../../../contexts/UserContext";
import AccountContext from "../../../contexts/AccountContext";

const mapBoxToken = process.env.NEXT_PUBLIC_MAPBOX;
const provider = new MapBoxProvider({
    params: {
        access_token: mapBoxToken,
    },
});

const searchControl = new GeoSearchControl({
    provider: provider,
    showMarker: true,
    showPopup: false,
    marker: {
        // optional: L.Marker    - default L.Icon.Default
        // icon: pointers["pin"],
        draggable: true,
    },
    autoClose: false,
    autoComplete: true, // optional: true|false  - default true
    autoCompleteDelay: 1000, // optional: number      - default 250
    searchLabel: "Introdueix una direcció",
    popupFormat: ({ query, result }) => result.label, // optional: function    - default returns result label,
    resultFormat: ({ result }) => result.label,    
    maxSuggestions: 5,
    keepResult: true,
    maxMarkers: 1,
    updateMap: true,
});

const SearchField = () => {
    // const provider = new OpenStreetMapProvider();
    const map = useMap();
    const { user } = useUserContext();
    const { /*account,*/ setAccount } = AccountContext();

    useEffect(() => {
        map.addControl(searchControl);

        map?.on("geosearch/showlocation", (e) => {
            
            //Poder tancar-ho 
            let mark = e.marker
            let btn = document.createElement("a");
            btn.style.cursor = "pointer";
            btn.innerText = "🗑️";
            btn.onclick = function () {
                map.removeLayer(mark);
            };
           
            mark.bindPopup(btn, {
                maxWidth: "auto",
            });
            /////
            
            if (e?.location.raw) e.location.raw.action = "last";
            const token = user?.token ?? false;
            const url = `/api/data/directions`;
            fetch(url, {
                method: "PUT",
                headers: {
                    Accept: "application/json, text/plain, */*",
                    "Content-Type": "application/json",
                    token,
                },
                body: JSON.stringify({ data: e?.location?.raw }),
            })
                .then((resp) => resp.json())
                .then((data) => {
                    setAccount((prevState) => {
                        if (prevState) {
                            if(!prevState.directions){
                                prevState.directions = {};
                            }
                            prevState.directions.last = data?.data?.directions?.last;
                        }
                        return prevState;
                    });
                })
                .catch((err) => {});
        });
        map.on('geosearch/marker/dragend', function e(e){            
            e.event.target.remove()
        });

        return () => {
            map.off("geosearch/showlocation");
            map.off("geosearch/marker/dragend");
            map.removeControl(searchControl);
        };
    }, [map, user]);

    return null;
};

export default SearchField;
