import React, { useCallback, useRef, useState, useEffect } from "react";
import { Marker, Popup, Tooltip } from "react-leaflet";
import * as pointers from "./pointerIcon";
import Link from "next/link";
import ServeisContext from "../../../contexts/ServeisContext";
import { ModalServices } from "../../customModals/ModalServices/ModalServices";
import useUserContext from "../../../contexts/UserContext";
import { toast } from "react-toastify";

const mapBoxToken = process.env.NEXT_PUBLIC_MAPBOX;

const Servei = ({ servei }) => {
    const coordinates = servei?.geometry?.coordinates;
    const { user } = useUserContext();
    const { serveis, setserveis } = ServeisContext();
    const [position, setPosition] = useState([
        servei?.geometry?.coordinates[1],
        servei?.geometry?.coordinates[0],
    ]);
    const [show, setshow] = useState(false);
    const [draggable, setDraggable] = useState(false);

    const markerRef = useRef(null);
    const popupElRef = useRef(null);

    const convertDate = (date) => {
        if (date) {
            const d = new Date(date);
            const options = {
                year: "numeric",
                month: "2-digit",
                day: "2-digit",
                hour: "2-digit",
                minute: "2-digit",
                second: "2-digit",
            };
            return d.toLocaleDateString("es", options);
        } else {
            return false;
        }
    };
    const toggleDraggable = useCallback((e) => {
        popupElRef.current._closeButton.click();
        e.preventDefault();
        e.stopPropagation();
        setDraggable((prevState) => !prevState);
    }, []);

    const handleEdit = (e) => {
        e.preventDefault();
        e.stopPropagation();
        setshow(true);
    };

    const handleClose = (e) => {
        e.preventDefault();
        e.stopPropagation();
        const token = user?.token ?? false;
        const urlApi = `/api/data/services`;
        let formValues = {
            _id: servei?._id,
            title: servei?.title ?? "",
            direction: servei?.direction,
            status: "closed",
            objects: servei?.objects ?? [],
            dateFinish: Date.now(),
            lat: servei?.geometry?.coordinates[1],
            lon: servei?.geometry?.coordinates[0],
            service: servei?.service,
            result: servei?.result,
        };
        fetch(urlApi, {
            method: "PUT",
            headers: {
                Accept: "application/json, text/plain, */*",
                "Content-Type": "application/json",
                token,
            },
            body: JSON.stringify({ data: formValues }),
        })
            .then((resp) => resp.json())
            .then((d) => {
                if (!!d.accion) {
                    setserveis((prevState) => {
                        //Si hi ha serveis sino retorna directament
                        if (prevState?.length > 0) {
                            const index = prevState.findIndex(
                                (s) => s?._id === d?.data?._id
                            );
                            index >= 0
                                ? (prevState[index] = d?.data)
                                : prevState.push(d?.data);

                            return prevState;
                        }
                        prevState = [];
                        prevState.push(d?.data);

                        return prevState;
                    });
                    setshow(false);
                    toast.info("S'ha Tancat correctament!");
                } else {
                    toast.error("No s'ha pogut Tancar!");
                }
            })
            .catch((err) => {
                console.log(err);
                toast.error("No s'ha pogut Tancar!");
            });
    };

    useEffect(() => {
        setPosition([
            servei?.geometry?.coordinates[1],
            servei?.geometry?.coordinates[0],
        ]);
    }, [servei]);

    return (
        <Marker
            id={servei?._id}
            icon={pointers[`pin${servei?.status}`]}
            draggable={draggable}
            ref={markerRef}
            position={position}
            key={servei?._id}
            servei={servei}            
            eventHandlers={{
                dragend: (e) => {
                    // setDraggable(!draggable);

                    const url = `https://api.mapbox.com/geocoding/v5/mapbox.places/${e?.target?._latlng?.lng},${e?.target?._latlng?.lat}.json?access_token=${mapBoxToken}`;
                    const geocoding = () => {
                        fetch(url)
                            .then((resp) => resp.json())
                            .then((p) => {
                                if (p) {
                                    const data = serveis.find(
                                        (serv) =>
                                            serv._id == e?.target?.options?.id
                                    );
                                    data.direction = p?.features[0]?.place_name;
                                    data.lon =
                                        e?.target?._latlng?.lng?.toFixed(6);
                                    data.lat =
                                        e?.target?._latlng?.lat?.toFixed(6);

                                    const token = user?.token ?? false;
                                    const urlApi = `/api/data/services`;
                                    const SendData = async () => {
                                        fetch(urlApi, {
                                            method: "PUT",
                                            headers: {
                                                Accept: "application/json, text/plain, */*",
                                                "Content-Type":
                                                    "application/json",
                                                token,
                                            },
                                            body: JSON.stringify({ data }),
                                        })
                                            .then((resp) => resp.json())
                                            .then((d) => {
                                                if (!!d.accion) {
                                                    setserveis((prevState) => {
                                                        //Si hi ha serveis sino retorna directament
                                                        if (
                                                            prevState?.length >
                                                            0
                                                        ) {
                                                            const index =
                                                                prevState.findIndex(
                                                                    (servei) =>
                                                                        servei?._id ===
                                                                        d?.data
                                                                            ?._id
                                                                );
                                                            if (index >= 0) {
                                                                prevState[
                                                                    index
                                                                ] = d?.data;
                                                            } else {
                                                                prevState.push(
                                                                    d?.data
                                                                );
                                                            }
                                                            return prevState;
                                                        }
                                                        prevState = [];
                                                        prevState.push(d?.data);

                                                        return prevState;
                                                    });
                                                    setshow(false);
                                                    toast.info(
                                                        "S'ha actualitzat correctament!"
                                                    );
                                                } else {
                                                    toast.error(
                                                        "No s'ha pogut actualitzar!"
                                                    );
                                                }
                                            })
                                            .catch((err) => {
                                                toast.error(
                                                    "No s'ha pogut actualitzar!"
                                                );
                                            });
                                    };
                                    SendData();
                                }
                            })
                            .catch((err) => {
                                toast.error("Error recuperant l'adreça");
                            });
                    };
                    geocoding();
                },
            }}
        >
            <Tooltip
                // className={`tipVisible`}
                direction="top"
                sticky
                offset={[0, 0]}
                opacity={1}
                // permanent
            >
                {servei?.title}
            </Tooltip>
            <Popup ref={popupElRef}>
                <table>
                    <tbody>
                        <tr>
                            <td style={{ minWidth: "70px" }}>
                                <strong>Títol:</strong>
                            </td>
                            <td>
                                <strong>{servei?.title?.toUpperCase()}</strong>
                            </td>
                        </tr>
                        <tr>
                            <td>
                                <strong>Inici:</strong>
                            </td>
                            <td>{convertDate(servei?.dateInput)}</td>
                        </tr>
                        {servei?.dateAssign && (
                            <tr>
                                <td>
                                    <strong>Traspàs:</strong>
                                </td>
                                <td>{convertDate(servei?.dateAssign)}</td>
                            </tr>
                        )}
                        <tr>
                            <td>
                                <strong>Posició:</strong>
                            </td>
                            <td>
                                <Link
                                    href={`https://maps.google.com/maps?q=${coordinates[1]},${coordinates[0]}&amp;t=m`}
                                >
                                    <a
                                        target="_blank"
                                        rel="noreferrer"
                                        title="Gooble Maps"
                                    >
                                        {coordinates[1].toFixed(5)}
                                        °,
                                        {coordinates[0].toFixed(5)}°
                                    </a>
                                </Link>
                                {`-----`}
                                <Link
                                    href={`https://www.google.com/maps?q&layer=c&cbll=${coordinates[1]},${coordinates[0]}`}
                                >
                                    <a target="_blank" rel="noreferrer">
                                        {"Street View"}
                                    </a>
                                </Link>
                            </td>
                        </tr>
                        {servei?.direction && (
                            <tr>
                                <td>
                                    <strong>Direcció:</strong>
                                </td>
                                <td>{servei?.direction}</td>
                            </tr>
                        )}
                        {servei?.service && (
                            <tr>
                                <td>
                                    <strong>Servei:</strong>
                                </td>
                                <td>{servei?.service}</td>
                            </tr>
                        )}
                        {servei?.result && (
                            <tr>
                                <td>
                                    <strong>Resultat:</strong>
                                </td>
                                <td>{servei?.result}</td>
                            </tr>
                        )}
                        <tr>
                            <td>
                                <strong>Estat:</strong>
                            </td>
                            <td>{servei?.status}</td>
                        </tr>
                        {servei?.objects?.length > 0 && (
                            <tr>
                                <td>
                                    <strong>Vehicles:</strong>
                                </td>
                                <td>
                                    {servei?.objects?.length > 0 &&
                                        servei?.objects?.map((object) => {
                                            return (
                                                <span
                                                    key={object?.value}
                                                >{`-${object?.label}`}</span>
                                            );
                                        })}
                                </td>
                            </tr>
                        )}
                        <tr>
                            <td className="d-flex justify-content-end">📍</td>
                            <td>
                                <a href="" onClick={toggleDraggable}>
                                    {!draggable
                                        ? "Moure de posició"
                                        : "Blocar posició"}
                                </a>
                            </td>
                        </tr>
                        <tr>
                            <td className="d-flex justify-content-end">📝</td>
                            <td>
                                <a href="" onClick={handleEdit}>
                                    Editar
                                </a>
                            </td>
                        </tr>
                        <tr>
                            <td className="d-flex justify-content-end">🔒</td>
                            <td>
                                <a href="" onClick={handleClose}>
                                    Tancar servei
                                </a>
                            </td>
                        </tr>
                    </tbody>
                </table>
            </Popup>
            {show && (
                <ModalServices
                    setshow={setshow}
                    show={show}
                    title={"Edició d'un servei"}
                    isNew={false}
                    data={servei}
                />
            )}
        </Marker>
    );
};

export default Servei;
