import {
    Circle,
    FeatureGroup,
    LayersControl,
    Marker,
    Polygon,
    Polyline,
    Popup,
    Rectangle,
    useMap,
} from "react-leaflet";
import "leaflet-geosearch/dist/geosearch.css";
import { useEffect } from "react";
import useUserContext from "../../../contexts/UserContext";
import AccountContext from "../../../contexts/AccountContext";
import { EditControl } from "react-leaflet-draw";
import { useState } from "react";
import { ModalZona } from "../../customModals/ModalZona/ModalZona";

function Zona({ zona }) {
   
    switch (zona?.type) {
        case "polyline":
            return (
                <Polyline
                    pathOptions={{ color: `${zona?.color}` }}
                    positions={zona?.coordinates}
                >
                    <Popup>{zona?.name}</Popup>
                </Polyline>
            );
        case "polygon":
            return (
                <Polygon
                    pathOptions={{ color: `${zona?.color}` }}
                    // fillColor={zona?.color}
                    positions={zona?.coordinates}
                >
                    <Popup>{zona?.name}</Popup>
                </Polygon>
            );
        case "rectangle":
            return (
                <Rectangle
                    bounds={zona?.coordinates}
                    pathOptions={{ color: `${zona?.color}` }}
                >
                    <Popup>{zona?.name}</Popup>
                </Rectangle>
            );
        case "circle":           
            return (
                <>
                    <Circle
                        pathOptions={{ fillColor: `${zona?.color}` }}
                        center={zona?.coordinates[0]}
                        radius={zona?.radius}
                        // eventHandlers={{
                        //     click: (e) => {
                        //         console.log(e);
                        //     },
                        // }}
                    >
                        <Popup>{zona?.name}</Popup>
                    </Circle>
                </>
            );
        default:
            break;
    }

    return null;
}

function Mark({ mark }) {
    try {
        return (
            <Marker position={mark?.coordinates[0]}>
                <Popup>{mark?.name}</Popup>
            </Marker>
        );
    } catch (error) {
        return null;
    }
}

const Zones = ({ zones }) => {
   
    let allZones = [];
    let allMarks = [];

    if (zones) {
        if (zones?.entity?.polygons?.length > 0) {
            allZones = allZones.concat(zones.entity.polygons);
        }
        if (zones?.main?.polygons?.length > 0) {
            allZones = allZones.concat(zones.main.polygons);
        }
        if (zones?.entity?.marks?.length > 0) {
            allMarks = allMarks.concat(zones.entity.marks);
        }
        if (zones?.main?.marks?.length > 0) {
            allMarks = allMarks.concat(zones.main.marks);
        }
    }

    return (
        <>
            <FeatureGroup>
                <EditControl
                    draw={{
                        marker: false,
                        circlemarker: false,
                        polyline: false,
                        polygon: false,
                        circle: false,
                        rectangle: false,
                    }}
                    edit={{ edit: false, remove: false }}
                />
                {allZones?.length &&
                    allZones.map((zona) => {
                        if(zona?.visible){
                            return <Zona key={zona._id} zona={zona} />;
                        }else{return null}
                    })}
                {allMarks?.length &&
                    allMarks.map((m) => {
                        if(m?.visible){
                            return <Mark key={m._id} mark={m} />;

                        }else{return null}
                    })}
                {/* 
                
                 */}
            </FeatureGroup>
        </>
    );
};

export default Zones;
