/**
 * A partir del SlideMenuContext es recullen dos parametres, el
 * numero de item presionat (itemNav) i la acció que porta aparellada
 * el botó pressionat. Si l'acció es toogle, fa que es replegui el menú
 * i torni a sortir amb el nou, si l'acció és diferent replega el menú i no
 * el fa tornar a sortir. L'accio de sortir o tancar-se del menú es fa amb
 * les classes showSideNav i hiddeSideNav, que s'alternen entre elles.
 */

/* eslint-disable @next/next/no-img-element */
import React, { useEffect, useRef, useState } from "react";
import slideMenuContext from "../../../contexts/SlideMenuContext";
import CheckEvents from "./Slides/Events/CheckEvents";
import Events from "./Slides/Events/Events";
import Llocs from "./Slides/Llocs/Llocs";
import Serveis from "./Slides/Serveis/Serveis";

import VehiclesSlide from "./Slides/Vehicles/VehiclesSlide";

const SlideMenu = () => {
    const { itemNav, setItemNav, action, setaction } = slideMenuContext();
    const slideMenu = useRef();
    const [events, setevents] = useState([]);

    useEffect(() => {
        //Acció diferent a tancar
        if (itemNav != -1) {
            //Primer de tot treu les due classes per si alguna esta
            slideMenu.current.classList.remove("showSideNav");
            slideMenu.current.classList.remove("hiddeSideNav");
            //Si es toggle l'acció que surti el menú
            if (action === "toggle") {
                setTimeout(() => {
                    slideMenu.current.classList.add("showSideNav");
                }, 300);
                //Si no es toggle l'acció que indiqui que el menú esta tancat.
            } else {
                slideMenu.current.classList.replace(
                    "showSideNav",
                    "hiddeSideNav"
                );
            }
            //Acció de tancar menú
        } else if (itemNav === -1) {
            //Si esta obert el tanca.
            slideMenu.current.classList.replace("showSideNav", "hiddeSideNav");
            //Treu highligth blau del boto al tancar el menú.
            document
                .getElementsByClassName("active")[0]
                ?.classList.toggle("active");
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [itemNav]);

    return (
        <div id="slideMenu" className="sideNav" ref={slideMenu}>
            <div
                className="closeDiv"
                onClick={() => {
                    setItemNav(-1);
                    setaction("");
                }}
            >
                <span className="spanClose">Tancar</span>

                <img
                    className="imgClose"
                    src="/iconClose.svg"
                    alt="Aspa de tancament"
                />
            </div>
            <CheckEvents setevents={setevents} events={events} />
            <div className="containerMenu">
                {itemNav == 0 && <VehiclesSlide />}
                {itemNav == 1 && <Serveis />}
                {itemNav == 2 && <Llocs />}
                {itemNav == 3 && <Events events={events}/>}
            </div>
            <style jsx>{`
                .sideNav {
                    width: 22rem;
                    position: absolute;
                    height: 100%;
                    box-shadow: 0.125rem 0.3rem 0.325rem #888;
                    left: -50rem;
                    z-index: 2;
                    overflow: auto;
                    background: var(--background);
                }
                .showSideNav {
                    animation-name: showSide;
                    animation-duration: 0.3s;
                    animation-fill-mode: forwards;
                }

                .hiddeSideNav {
                    animation-name: hiddeSide;
                    animation-duration: 0.8s;
                    animation-fill-mode: forwards;
                }

                .containerMenu {
                    height: 90vh;
                    padding: 15px;
                }

                @keyframes showSide {
                    from {
                        left: -6rem;
                    }
                    to {
                        left: 5rem;
                    }
                }

                @keyframes hiddeSide {
                    from {
                        left: 5rem;
                    }
                    to {
                        left: -50rem;
                    }
                }

                .closeDiv {
                    font-size: 1.3rem;
                    font-weight: 600;
                    padding: 0.5rem 1rem;
                    text-align: left;
                    cursor: pointer;
                    background: var(--tileLigth);
                    height: 3rem;
                }
                .closeDiv :hover {
                    background: var(--primaryBlue);
                    color: var(--white);
                }
                .spanClose {
                    user-select: none;
                }

                .closeDiv > img {
                    margin-left: 0.5rem;
                    height: 2rem;
                    user-select: none;
                }

                /*Landscape & Portrait*/
                @media (max-width: 600px), (max-height: 600px) {
                    .sideNav {
                        margin-left: -5rem;
                        z-index: 10;
                        height: 110%;
                    }
                    .showSideNav {
                        width: 100%;
                    }
                    .hiddeSideNav {
                        width: 0;
                    }
                }
            `}</style>
        </div>
    );
};

export default SlideMenu;
