import { OverlayTrigger, Popover, Tab, Tabs } from "react-bootstrap";
import AccountContext from "../../../../../contexts/AccountContext";
import MapContext from "../../../../../contexts/MapContext";
import useUserContext from "../../../../../contexts/UserContext";

function Direction({ direction, fav }) {
    const { user } = useUserContext();
    const { /*account,*/ setAccount } = AccountContext();

    const coordinates = [
        direction?.geometry?.coordinates[1],
        direction?.geometry?.coordinates[0],
    ];
    const closePopover = (e) => {
        document.getElementsByTagName(`body`)[0].click();
    };
    const { map /*setmap*/ } = MapContext();

    const handleViewMap = () => {
        map.flyTo(coordinates, 19);
        let mp = new L.Marker(
            coordinates,
            // { icon: pointers['pin'] },
            {
                draggable: "true",
            }
        );
        mp.addTo(map);

        let btn = document.createElement("a");
        btn.style.cursor = "pointer";
        btn.innerText = "🗑️";
        btn.onclick = function () {
            map.removeLayer(mp);
        };
        mp.bindTooltip(direction?.place_name, {
            direction: "right",
        });
        mp.bindPopup(btn, {
            maxWidth: "auto",
        });
    };

    const handleFAVclick = () => {
        const token = user?.token ?? false;
        const url = `/api/data/directions`;

        if (direction) direction.action = "fav";

        fetch(url, {
            method: "PUT",
            headers: {
                Accept: "application/json, text/plain, */*",
                "Content-Type": "application/json",
                token,
            },
            body: JSON.stringify({ data: direction }),
        })
            .then((resp) => resp.json())
            .then((data) => {
                setAccount((prevState) => {
                    console.log(prevState);
                    if (prevState) {
                        prevState.directions.fav = data?.data?.directions?.fav;
                    }
                    return prevState;
                });
            })
            .catch((err) => {});
    };

    const handleDELclick = (type) => {
        const token = user?.token ?? false;
        const url = `/api/data/directions`;

        if (direction) direction.action = type;

        fetch(url, {
            method: "DELETE",
            headers: {
                Accept: "application/json, text/plain, */*",
                "Content-Type": "application/json",
                token,
            },
            body: JSON.stringify({ data: direction }),
        })
            .then((resp) => resp.json())
            .then((data) => {
                setAccount((prevState) => {
                    console.log(prevState);
                    if (prevState) {
                        prevState.directions[`${type}`] =
                            data?.data?.directions?.[type];
                    }
                    return prevState;
                });
            })
            .catch((err) => {});
    };

    const popover = (
        <Popover>
            <Popover.Title as="h3" className="text-center">
                Accions
            </Popover.Title>
            <Popover.Content
                onClick={() => {
                    handleViewMap();
                    closePopover();
                }}
            >
                Centrar al mapa
            </Popover.Content>

            {!fav && (
                <>
                    <Popover.Content
                        onClick={() => {
                            handleFAVclick();
                            closePopover();
                        }}
                    >
                        Agregar a FAV&rsquo;s
                    </Popover.Content>
                    <Popover.Content
                        onClick={() => {
                            handleDELclick("last");
                            closePopover();
                        }}
                    >
                        Eliminar
                    </Popover.Content>
                </>
            )}
            {fav && (
                <Popover.Content
                    onClick={() => {
                        handleDELclick("fav");
                        closePopover();
                    }}
                >
                    Treure de FAV&rsquo;s
                </Popover.Content>
            )}
        </Popover>
    );

    return (
        <tr key={direction._id}>            
            <td className="center overfloow" style={{ display: "inline-flex" }}>
                <img
                    src="/miscelanea/pin.svg"
                    width={15}
                    height={15}
                    style={{ margin: "2px 0px 0px 0px" }}
                ></img>
                <div
                    onClick={handleViewMap}
                    title={direction?.place_name}
                    className="overfloow"
                    style={{
                        cursor: "pointer",
                        textAlign: "left",
                    }}
                >
                    {direction?.place_name}
                </div>
            </td>
            <td className="center pointer">
                <OverlayTrigger
                    placement={
                        window.matchMedia("(max-width: 600px)").matches ||
                        window.matchMedia("(max-height: 600px)").matches
                            ? "left"
                            : "right"
                    }
                    trigger="click"
                    overlay={popover}
                    rootClose
                >
                    <img
                        title="Accions"
                        className="dotImg"
                        src="/navLateral/slides/dots.svg"
                        alt="Icona de menys"
                    />
                </OverlayTrigger>
            </td>
            <style jsx>{`
                .overfloow {
                    overflow: hidden;
                    white-space: nowrap;
                    text-overflow: ellipsis;
                }
                .pointer {
                    cursor: pointer;
                }
                .center {
                    align-self: center;
                    text-align: center;
                }
                .dotImg {
                    margin: 0px;
                    width: 16px;
                    height: 16px;
                }
            `}</style>
        </tr>
    );
}

const Directions = () => {
    const { account /*,setaccount*/ } = AccountContext();

    return (
        <div>
            <table>
                <thead>
                    <tr>
                        <th className="center"></th>
                        <th className="titleGroup center">
                            <div style={{ textAlign: "right" }}>
                                {`Ultimes direccions (${account?.directions?.last?.length || 0})`}
                            </div>
                        </th>
                        <th className="center pointer"></th>
                    </tr>
                </thead>
                {account?.directions?.last?.length > 0 && (
                    <tbody>
                        {account?.directions?.last?.map((direction) => {
                            return (
                                <Direction
                                    key={direction?._id}
                                    direction={direction}
                                    fav={false}
                                />
                            );
                        })}
                    </tbody>
                )}
            </table>
            <table>
                <thead>
                    <tr>
                        <th className="center"></th>
                        <th className="titleGroup center">
                            <div style={{ textAlign: "right" }}>
                                {`Direccions preferides (${account?.directions?.fav?.length || 0})`}
                            </div>
                        </th>
                        <th className="center pointer"></th>
                    </tr>
                </thead>
                {account?.directions?.fav?.length > 0 && (
                    <tbody>
                        {account?.directions?.fav?.map((direction) => {
                            return (
                                <Direction
                                    key={direction?._id}
                                    direction={direction}
                                    fav={true}
                                />
                            );
                        })}
                    </tbody>
                )}
            </table>

            <style jsx>{`
                :global(.popover-body) {
                    border-bottom: 1px #8a8a8a90 solid;
                    cursor: pointer;
                }

                .pointer {
                    cursor: pointer;
                }
                .titleGroup {
                    font-size: normal;
                }
                .center {
                    align-self: center;
                    text-align: center;
                }

                table {
                    background: #007bff3e;
                    margin-top: 1.2rem;
                    width: 100%;
                    font-size: small;
                    margin-bottom: 2rem;
                }

                thead > tr {
                    display: grid;
                    grid-template-columns: 8% 88% 4%;
                    margin-top: 10px;
                }
                tbody > :global(tr: first-child) {
                    border-top: 1px #8a8a8a90 solid;
                }
                tbody > :global(tr) {
                    margin-left: 5px;
                    border-bottom: 1px #8a8a8a90 solid;
                    height: 2.5rem;
                    display: grid;
                    grid-template-columns: 92% 8%;
                }
                .hidden {
                    visibility: collapse;
                }
                .plus {
                    content: url("/navLateral/slides/plus.svg");
                }

                td > input {
                    margin-top: 5px;
                }
                td > div {
                    margin-left: 2px;
                }
            `}</style>
        </div>
    );
};

export default Directions;
