import { useState } from "react";
import { Modal, OverlayTrigger, Popover, Tab, Tabs } from "react-bootstrap";
import { toast } from "react-toastify";
import AccountContext from "../../../../../contexts/AccountContext";
import MapContext from "../../../../../contexts/MapContext";
import useUserContext from "../../../../../contexts/UserContext";
import ZonesContext from "../../../../../contexts/ZonesContext";
import { Button } from "../../../../globals/Button";

function Zone({ zone }) {
    const { user } = useUserContext();
    const { /*account,*/ setAccount } = AccountContext();
    const { zones, setzones } = ZonesContext();
    const closePopover = (e) => {
        document.getElementsByTagName(`body`)[0].click();
    };
    const { map /*setmap*/ } = MapContext();

    const handleViewMap = () => {
        map.flyTo(zone?.coordinates[0], 19);
    };

    const [show, setshow] = useState(false);

    const handleRemoveClick = () => {
        setshow(true);
    };

    const confirmRemoveZone = () => {
        const token = user?.token ?? false;
        const url = `/api/data/zones`;

        fetch(url, {
            method: "DELETE",
            headers: {
                Accept: "application/json, text/plain, */*",
                "Content-Type": "application/json",
                token,
            },
            body: JSON.stringify({ data: zone }),
        })
            .then((resp) => resp.json())
            .then((data) => {
                console.log(data)
                if (!!data.accion) {                    
                    setzones((prevState) => {
                        const type = (data?.data?.idAccount == user.id) ? "main" : "entity";
                        console.log(data?.data?.idAccount == user.id)
                        
                        const index = prevState[type]?.polygons.findIndex(
                            (s) => {
                                return s?._id == data?.data?._id}
                                );                        
                                if (index > -1) {
                                    prevState[type]?.polygons?.splice(index, 1);
                                }
                                return prevState;
                            });
                            toast.info("S'ha actualitzat correctament!");
                } else {
                    toast.error("No s'ha pogut actualitzar!");
                }
                
            })
            .catch((err) => {});
    };

    // main.polygons[0].visible
    const handleChangeChk = ({ target }) => {
        setzones((prevState) => ({
            main: {
                ...prevState.main,
                polygons: prevState.main.polygons.length
                    ? prevState.main.polygons.map((el) => {
                          if (el._id === zone._id) {
                              return {
                                  ...el,
                                  visible: target.checked,
                              };
                          } else {
                              return el;
                          }
                      })
                    : prevState.main.polygons,
            },
            entity: {
                ...prevState.entity,
                polygons: prevState.entity.polygons.length
                    ? prevState.entity.polygons.map((el) => {
                          if (el._id === zone._id) {
                              return {
                                  ...el,
                                  visible: target.checked,
                              };
                          } else {
                              return el;
                          }
                      })
                    : prevState.entity.polygons,
            },
        }));
    };

    const popover = (
        <Popover>
            <Popover.Title as="h3" className="text-center">
                Accions
            </Popover.Title>
            <Popover.Content
                onClick={() => {
                    handleViewMap();
                    closePopover();
                }}
            >
                Centrar al mapa
            </Popover.Content>
            {/* <Popover.Content
                onClick={() => {
                    // handleDELclick("last");
                    closePopover();
                }}
            >
                Editar
            </Popover.Content> */}
            <Popover.Content
                onClick={() => {
                    handleRemoveClick();
                    closePopover();
                }}
            >
                Eliminar
            </Popover.Content>
        </Popover>
    );

    return (
        <tr key={zone._id}>
            <td className="center">
                <input
                    checked={zone?.visible}
                    title="Veure"
                    className="pointer center"
                    type="checkbox"
                    style={{ textAlign: "center" }}
                    onChange={handleChangeChk}
                />
            </td>
            <td className="center overfloow" style={{ display: "inline-flex" }}>
                <span
                    className={`${zone?.type} draw`}
                    title={`${zone?.type}`}
                />
                <div
                    onClick={handleViewMap}
                    title={zone?.name}
                    className="overfloow"
                    style={{
                        cursor: "pointer",
                        textAlign: "left",
                    }}
                >
                    {` ${zone?.name} ― ${zone?.description}`}
                </div>
            </td>
            <td className="center pointer">
                <OverlayTrigger
                    placement={
                        window.matchMedia("(max-width: 600px)").matches ||
                        window.matchMedia("(max-height: 600px)").matches
                            ? "left"
                            : "right"
                    }
                    trigger="click"
                    overlay={popover}
                    rootClose
                >
                    <img
                        title="Accions"
                        className="dotImg"
                        src="/navLateral/slides/dots.svg"
                        alt="Icona de menys"
                    />
                </OverlayTrigger>
            </td>

            <Modal
                show={show}
                onHide={() => setshow(false)}
                backdrop="static"
                animation={false}
                size="lg"
                aria-labelledby="contained-modal-title-vcenter"
                scrollable
            >
                <Modal.Header closeButton>
                    <Modal.Title>Suprimir zona</Modal.Title>
                </Modal.Header>
                <Modal.Body>Esteu segur que el voleu suprimir?</Modal.Body>
                <Modal.Footer>
                    <Button
                        variant="primary"
                        onClick={() => {
                            confirmRemoveZone();
                            setshow(false);
                        }}
                    >
                        Suprimir
                    </Button>

                    <Button variant="secondary" onClick={() => setshow(false)}>
                        Cancelar
                    </Button>
                </Modal.Footer>
            </Modal>
            <style jsx>{`
                .draw {
                    background-image: url(/miscelanea/spritesDraw.png);
                    background-repeat: no-repeat;
                    background-size: 300px 30px;
                    -webkit-background-clip: padding-box;
                    background-clip: padding-box;
                    width: 22px;
                    height: 16px;
                }
                .polygon {
                    background-position: -35px -5px;
                }
                .polyline {
                    background-position: -5px -5px;
                }
                .rectangle {
                    background-position: -65px -5px;
                }
                .circle {
                    background-position: -95px -5px;
                }
                .overfloow {
                    overflow: hidden;
                    white-space: nowrap;
                    text-overflow: ellipsis;
                }
                .pointer {
                    cursor: pointer;
                }
                .center {
                    align-self: center;
                    text-align: center;
                }
                .dotImg {
                    margin: 0px;
                    width: 16px;
                    height: 16px;
                }
            `}</style>
        </tr>
    );
}

const Zones = () => {
    const { zones, setzones } = ZonesContext();

    return (
        <div>
            <table>
                <thead>
                    <tr>
                        <td className="borderRight center">
                            |
                            <img
                                src="/navLateral/slides/eye.svg"
                                alt="Icona d'ulls"
                                title="Veure"
                            />
                            |
                        </td>
                        <th className="titleGroup center">
                            <div style={{ textAlign: "right" }}>
                                {`Zones pròpies (${zones?.main?.polygons?.length})`}
                            </div>
                        </th>
                        <th className="center pointer"></th>
                    </tr>
                </thead>
                {zones?.main?.polygons?.length > 0 && (
                    <tbody>
                        {zones?.main?.polygons?.map((zone) => {
                            return (
                                <Zone zone={zone} key={zone?._id} main={true} />
                            );
                        })}
                    </tbody>
                )}
            </table>
            <table>
                <thead>
                    <tr>
                        <th className="center"></th>
                        <th className="titleGroup center">
                            <div style={{ textAlign: "right" }}>
                                {`Zones generals (${zones?.entity?.polygons?.length})`}
                            </div>
                        </th>
                        <th className="center pointer"></th>
                    </tr>
                </thead>
                {zones?.entity?.polygons?.length > 0 && (
                    <tbody>
                        {zones?.entity?.polygons?.map((zone) => {
                            return (
                                <Zone zone={zone} main={true} key={zone?._id} />
                            );
                        })}
                    </tbody>
                )}
            </table>

            <style jsx>{`
                :global(.popover-body) {
                    border-bottom: 1px #8a8a8a90 solid;
                    cursor: pointer;
                }

                .pointer {
                    cursor: pointer;
                }
                .titleGroup {
                    font-size: normal;
                }
                .center {
                    align-self: center;
                    text-align: center;
                }

                table {
                    background: #007bff3e;
                    margin-top: 1.2rem;
                    width: 100%;
                    font-size: small;
                    margin-bottom: 2rem;
                }

                thead > tr {
                    display: grid;
                    grid-template-columns: 10% 88%;
                    margin-top: 10px;
                }
                thead > tr > td > img {
                    height: 20px;
                }
                tbody > :global(tr: first-child) {
                    border-top: 1px #8a8a8a90 solid;
                }
                tbody > :global(tr) {
                    border-bottom: 1px #8a8a8a90 solid;
                    height: 2.5rem;
                    display: grid;
                    grid-template-columns: 10% 85% 5%;
                }
                .hidden {
                    visibility: collapse;
                }
                .plus {
                    content: url("/navLateral/slides/plus.svg");
                }

                td > input {
                    margin-top: 5px;
                }
                td > div {
                    margin-left: 2px;
                }
            `}</style>
        </div>
    );
};

export default Zones;
