import Image from "next/image";
import { useState } from "react";
import MapContext from "../../../../../contexts/MapContext";
import { ModalServices } from "../../../../customModals/ModalServices/ModalServices";
import {
    convertDate,
    dateToHour,
    onlyDate,
} from "../../../../../utils/lib/dates";
import { toast, ToastContainer } from "react-toastify";
import useUserContext from "../../../../../contexts/UserContext";
import ServeisContext from "../../../../../contexts/ServeisContext";

const Servei = ({ servei }) => {
    const [open, setopen] = useState(false);
    const [show, setshow] = useState(false);
    const { map } = MapContext();
    const { user } = useUserContext();
    const { /*serveis,*/ setserveis } = ServeisContext();

    const handleClick = () => setopen((prevState) => !prevState);
    const handleClickPos = () => {
        const lat = servei?.geometry.coordinates[1];
        const lon = servei?.geometry.coordinates[0];
        map.flyTo([lat, lon], 18);
    };
    const handleEdit = (e) => {
        e.preventDefault();
        e.stopPropagation();
        setshow(true);
    };

    const handleClose = (e) => {
        e.preventDefault();
        e.stopPropagation();
        const token = user?.token ?? false;
        const urlApi = `/api/data/services`;
        let formValues = {
            _id: servei?._id,
            title: servei?.title ?? "",
            direction: servei?.direction,
            status: "closed",
            objects: servei?.objects ?? [],
            dateFinish: Date.now(),
            lat: servei?.geometry?.coordinates[1],
            lon: servei?.geometry?.coordinates[0],
            service: servei?.service,
            result: servei?.result,
        };
        fetch(urlApi, {
            method: "PUT",
            headers: {
                Accept: "application/json, text/plain, */*",
                "Content-Type": "application/json",
                token,
            },
            body: JSON.stringify({ data: formValues }),
        })
            .then((resp) => resp.json())
            .then((d) => {
                if (!!d.accion) {
                    setserveis((prevState) => {
                        //Si hi ha serveis sino retorna directament
                        if (prevState?.length > 0) {
                            const index = prevState.findIndex(
                                (s) => s?._id === d?.data?._id
                            );
                            if (index >= 0) {
                                prevState[index] = d?.data;
                            } else {
                                prevState.push(d?.data);
                            }
                            return prevState;
                        }
                        prevState = [];
                        prevState.push(d?.data);

                        return prevState;
                    });
                    setshow(false);
                    toast.info("S'ha Tancat correctament!");
                } else {
                    toast.error("No s'ha pogut Tancar!");
                }
            })
            .catch((err) => {
                console.log(err);
                toast.error("No s'ha pogut Tancar!");
            });
    };

    return (
        <>
            {!open && (
                <div className="card" key={servei._id}>
                    <div
                        className={`card-title ${servei?.status}`}
                        onClick={handleClick}
                    >
                        <div style={{ margin: "5px 0px" }}>
                            <h4 className="title">{servei?.title}</h4>
                        </div>
                    </div>
                    <div className="card-time">
                        <span className="time-init" title="Hora inici">
                            ☎ {dateToHour(servei?.dateInput)}h
                        </span>
                        {servei?.dateAssign && (
                            <span
                                className="time-transfer"
                                title="Hora traspàs"
                            >
                                📣 {dateToHour(servei?.dateAssign)}h
                            </span>
                        )}
                        {servei?.dateFinish && (
                            <span
                                className="time-finish"
                                title="Hora tancament"
                            >
                                🔒 {dateToHour(servei?.dateFinish)}h
                            </span>
                        )}
                    </div>
                    <div
                        className="card-place pointer"
                        onClick={handleClickPos}
                    >
                        <small style={{ margin: "-3px 0px -8px 0px" }}>
                            {onlyDate(servei?.dateInput)}
                        </small>
                        <span className="direction" title={servei?.direction}>
                            {servei?.direction}
                        </span>
                    </div>
                </div>
            )}
            {open && (
                <div className="card" key={servei._id}>
                    <div
                        className={`card-title ${servei?.status}`}
                        onClick={handleClick}
                    >
                        <div style={{ margin: "5px 0px" }}>
                            <h4 className="title">{servei?.title}</h4>
                        </div>
                    </div>
                    <div className="card-time">
                        <span className="time-init" title="Hora inici">
                            ☎ {dateToHour(servei?.dateInput)}h
                        </span>

                        {servei?.dateAssign && (
                            <span
                                className="time-transfer"
                                title="Hora traspàs"
                            >
                                📣 {dateToHour(servei?.dateAssign)}h
                            </span>
                        )}
                        {servei?.dateFinish && (
                            <span
                                className="time-finish"
                                title="Hora tancament"
                            >
                                🔒 {dateToHour(servei?.dateFinish)}h
                            </span>
                        )}
                    </div>
                    <div
                        className="card-place pointer"
                        onClick={handleClickPos}
                        style={
                            open
                                ? { borderBottom: "1px solid var(--tileLigth)" }
                                : {}
                        }
                    >
                        <small style={{ margin: "-3px 0px -8px 0px" }}>
                            {onlyDate(servei?.dateInput)}
                        </small>
                        <div className="pointer" style={{ marginTop: "10px" }}>
                            <Image
                                src={`/miscelanea/gps.svg`}
                                alt="imatge logo gps"
                                height="15px"
                                width="15px"
                                title="Direcció del Servei"
                            ></Image>
                        </div>
                        <span
                            className="direction"
                            title={servei?.direction}
                            style={{ marginTop: "-10px" }}
                        >
                            {servei?.direction}
                        </span>
                    </div>
                    <div className="card-service">
                        <span>{servei?.service}</span>
                    </div>
                    <div className="card-result">
                        <span>{servei?.result}</span>
                    </div>
                    <div className={`card-stats ${servei?.status}`}>
                        <div className="stat pointer">
                            <div className="value">Vehicles</div>
                            <div className="type">
                                {servei?.objects?.length > 0 &&
                                    servei?.objects?.map((object) => {
                                        return (
                                            <span key={object?.value}>
                                                {" "}
                                                -{object?.label}
                                            </span>
                                        );
                                    })}
                            </div>
                        </div>
                        <div className="stat">
                            <div className="value">Estat</div>
                            <div className="type">{servei?.status}</div>
                        </div>
                        <div className="stat">
                            <div
                                className="type pointer"
                                title="Editar"
                                onClick={handleEdit}
                            >
                                &#128221;
                            </div>
                        </div>
                        {servei?.status !== "closed" && (
                            <div className="stat">
                                <div
                                    className="type pointer"
                                    title="Tancar"
                                    onClick={handleClose}
                                >
                                    &#128274;
                                </div>
                            </div>
                        )}
                    </div>
                </div>
            )}
            {show && (
                <ModalServices
                    setshow={setshow}
                    show={show}
                    title={"Edició d'un servei"}
                    isNew={false}
                    data={servei}
                />
            )}
            <style jsx>{`
                .card {
                    margin-top: 10px;
                    display: grid;
                    grid-template-areas: "title" "time" "place" "service" "result" "stats";
                    border-radius: 18px;
                    background: white;
                    box-shadow: 0px 0px 3px #1343de;
                    font-family: "system-ui, apple-system, BlinkMacSystemFont, Segoe UI,        Roboto, Oxygen, Ubuntu, Cantarell, Fira Sans, Droid Sans,Helvetica Neue, sans-serif";
                    text-align: center;
                    border: 0px solid rgba(0, 0, 0);
                }
                .card-title {
                    grid-area: title;
                    margin-top: 0px;
                    border-top-left-radius: 15px;
                    border-top-right-radius: 15px;
                    margin-bottom: 5px;
                    cursor: pointer;
                }
                .title {
                    margin-bottom: 0px;
                }
                .card-time {
                    color: var(--tileLigth);
                    grid-area: time;
                    font-size: 13px;
                    display: grid;
                    grid-template-columns: 20% 20% 20% 20% 20%;
                }
                .time-init {
                    grid-column: 2;
                }
                .time-transfer {
                    grid-column: 3;
                }
                .time-finish {
                    grid-column: 4;
                }
                .card-place {
                    grid-area: place;

                    flex-direction: column;
                    display: inline-grid;
                }
                .direction {
                    overflow: hidden;
                    white-space: nowrap;
                    text-overflow: ellipsis;
                    margin: 1px 0px 5px 5px;
                }
                .card-service {
                    grid-area: service;
                    color: grey;
                    font-size: 15px;
                    font-weight: 300;
                    margin-top: 5px;
                    text-align: center;
                    line-height: 1.3;
                }
                .card-result {
                    grid-area: result;
                    font-size: 15px;
                    font-weight: bold;
                    text-align: center;
                    line-height: 1.3;
                    border-top: 5px dotted #bbb;
                    margin: 5px;
                    padding-top: 6px;
                }
                .card-stats {
                    grid-area: stats;
                    display: grid;
                    grid-template-columns: 2fr 1fr 1fr 1fr;
                    grid-template-rows: 1fr;
                    border-bottom-left-radius: 15px;
                    border-bottom-right-radius: 15px;
                }

                .card-stats .stat {
                    display: flex;
                    align-items: center;
                    justify-content: center;
                    flex-direction: column;

                    padding: 5px;
                }
                .card-stats .stat:nth-child(2) {
                    border-left: 1px solid;
                    border-right: 1px solid;
                }
                .card-stats .stat:nth-child(3) {
                    border-right: 1px solid;
                }
                .pointer {
                    cursor: pointer;
                }
                .open {
                    color: white;
                    background: var(--tileLigth);
                }
                .traspas {
                    background: var(--primaryBlue);
                    color: var(--textButton);
                }
                .closed {
                    background: var(--danger);
                    color: var(--textButton);
                }
            `}</style>
        </>
    );
};

export default Servei;
