import { Button } from "../../../../globals/Button";
import { GeneralModal } from "../../../../formulari/GeneralModal";
import { General } from "../../../../formulari/General";
import { useState } from "react";
import useUserContext from "../../../../../contexts/UserContext";
import { useForm } from "../../../../../Hooks/useForm";
import { Form } from "react-bootstrap";
import { toast } from "react-toastify";
import Servei from "./Servei";

import Spinner from "../../../../globals/Spinner";
import { colors } from "../../../../../styles/theme";

const Tabampliada = () => {
    const [formValidated, setformValidated] = useState(false);
    const [serveisAmpliats, setserveisAmpliats] = useState();
    const [loading, setloading] = useState(false);
    const { user } = useUserContext();
    const { token } = user ?? false;

    const changeInput = (e) => {
        handleInputChange(e);
        if (formValidated) {
            setformValidated(false);
        }
    };

    const handleSubmit = (e) => {
        e.preventDefault();
        e.stopPropagation();
        //Es carreguen al apretar submit per si hi ha
        //canvi de token en el inpass del modal
        const token = user?.token ?? false;
        const urlApi = `/api/data/services?`;

        const SendData = async () => {
            setloading(true)
            fetch(
                urlApi +
                    new URLSearchParams({
                        dInici: formValues?.dInici,
                        dFinal: formValues?.dFinal,
                    }).toString(),
                {
                    method: "GET",
                    headers: {
                        Accept: "application/json, text/plain, */*",
                        "Content-Type": "application/json",
                        token,
                    },
                }
            )
                .then((resp) => resp.json())
                .then((d) => {
                    setloading(false)
                    if (!!d.accion) {
                        setserveisAmpliats(d?.data);                                          
                    } else {
                        toast.error("No s'han pogut obtenir els serveis!");
                    }
                })
                .catch((err) => {
                    setloading(false)
                    console.log(err);
                    toast.error("No s'ha pogut consultar els serveis!");
                });
        };
        const form = e.currentTarget;
        //Comprovar la validesa del formulari
        form.checkValidity() ? SendData() : setformValidated(true);
    };

    const [formValues, handleInputChange, setValues /*,reset*/] = useForm({
        dInici: "",
        dFinal: "",
    });

    return (
        <>
            <Form noValidate validated={formValidated} onSubmit={handleSubmit}>
                <div className="inputsAmpli">
                    <General
                        changeInput={changeInput}
                        id={"titleService"}
                        label={"Data d'inici"}
                        name={"dInici"}
                        value={formValues?.dInici}
                        type={"datetime-local"}
                        required
                    />
                    <General
                        changeInput={changeInput}
                        id={"titleService"}
                        label={"Data final"}
                        name={"dFinal"}
                        value={formValues?.dFinal}
                        type={"datetime-local"}
                        required
                    />
                </div>
                <Form.Row
                    className="d-flex justify-content-center"
                    style={{ marginTop: "10px" }}
                >
                    <Button type="submit">Cercar</Button>
                </Form.Row>
            </Form>

            {serveisAmpliats?.length > 0 &&
              !loading &&
                serveisAmpliats?.map((servei) => {
                    return <Servei servei={servei} key={servei._id} />;
                })}
            {serveisAmpliats?.length <= 0 && (
                <h4>No hi ha cap servei per mostrar</h4>
            )}

            {loading && (
                <div
                    id="spinner"
                    style={{ visibility: "visible" }}
                    className="spinnerDiv"
                >
                    <Spinner colorspinner={colors.primaryBlue} />
                </div>
            )}

            <style jsx global>{`
            .inputsAmpli .form-group {
                    margin: -5px 0px -2px 0px;
                }
            `}</style>
        </>
    );
};

export default Tabampliada;
