import Image from "next/image";
import React, { useRef, useState } from "react";
import TableRow from "./TableRow";

const Table = ({ values, keyName, querySearch }) => {
    const bodyTable = useRef();
    const expandedIcon = useRef();
    const [plus, setplus] = useState(false);

    const handleClick = () => {
        bodyTable.current.classList.toggle("hidden");
        //expandedIcon.current.classList.toggle("plus");
        setplus(!plus);
    };

    //Ordenar les tables i els vehicles per nom
    values.sort(function (a, b) {
        if (a?.object?.name > b?.object?.name) {
            return 1;
        }
        if (a?.object?.name < b?.object?.name) {
            return -1;
        }
        // a must be equal to b
        return 0;
    });

    return (
        <table key={keyName}>
            <thead>
                <tr>
                    <th className="center">
                        {/* <input
                            className="pointer"
                            type="checkbox"
                            style={{ textAlign: "center" }}
                        /> */}
                    </th>
                    <th className="titleGroup center">
                        <div style={{ textAlign: "right" }}>
                            {keyName?.toUpperCase()}S ({values.length})
                        </div>
                    </th>
                    <th className="center pointer" onClick={handleClick}>
                        {plus ? (
                            <span className="noSelect">&#10133;</span> // + 
                            ) : (
                            <span className="noSelect">&#10134;</span>// - 
                        )}
                    </th>
                </tr>
            </thead>
            <tbody ref={bodyTable}>
                {values.map((unitat) => {
                    const o = unitat?.object;
                    if (
                        o?.name
                            .toUpperCase()
                            .includes(querySearch?.toUpperCase())
                    ) {
                        return <TableRow unitat={unitat} key={o._id} />;
                    }
                })}
            </tbody>

            <style jsx>{`
                .noSelect {
                    user-select: none;
                }

                th > img {
                    margin: 0px 0px;
                    width: 15px;
                    height: 15px;
                    pointer-events: none;
                    user-select: none;
                }
                .pointer {
                    cursor: pointer;
                }
                .titleGroup {
                    font-size: normal;
                }
                .center {
                    align-self: center;
                    text-align: center;
                }

                table {
                    background: #007bff3e;
                    margin-top: 1.2rem;
                    width: 100%;
                    font-size: x-small;
                    margin-bottom: 2rem;
                }
                
                thead > tr {
                    display: grid;
                    grid-template-columns: 8% 80% 12%;
                    margin-top: 10px;
                }
                tbody > :global(tr: first-child) {
                    border-top: 1px #8a8a8a90 solid;
                }
                tbody > :global(tr) {
                    border-bottom: 1px #8a8a8a90 solid;
                    height: 2.5rem;
                    display: grid;
                    grid-template-columns: 8% 8% 10% 35% 14% 10% 10% 5%;
                }
                .hidden {
                    visibility: collapse;
                }
                .plus {
                    content: url("/navLateral/slides/plus.svg");
                }
            `}</style>
        </table>
    );
};

export default Table;
