/* eslint-disable @next/next/no-img-element */

import React, { useState } from "react";

import ObjectsContext from "../../../../../contexts/ObjectsContext";
import Link from "next/link";

import "bootstrap/dist/css/bootstrap.css";
import OverlayTrigger from "react-bootstrap/OverlayTrigger";
import Popover from "react-bootstrap/Popover";
import { convertDate } from "../../../../../utils/lib/dates";
import MapContext from "../../../../../contexts/MapContext";
import { ModalSettings } from "../../../../customModals/ModalSettings/ModalSettings";
import { Menu, MenuItem } from "@szhsin/react-menu";
import "@szhsin/react-menu/dist/index.css";
import useUserContext from "../../../../../contexts/UserContext";
import { ModalAmpliada } from "../../../../customModals/ModalAmpliada/ModalAmpliada";

const iconStrengh = (gsm_strengh) => {
    if (!gsm_strengh || gsm_strengh < 0) {
        return 0;
    }
    if (gsm_strengh > 4) {
        return 4;
    }
    return gsm_strengh;
};

const closePopover = (e) => {
    document.getElementsByTagName(`body`)[0].click();
};

function LinkHistoric({ unitat, title, minutes }) {
    const { map } = MapContext();
    const { user } = useUserContext();
    const { token } = user ?? false;
    return (
        <Link
            href={{
                pathname: "/historic",
                query: {
                    token,
                    id: unitat?.object?._id,
                    minutes: minutes,
                    lat: map?.getCenter().lat?.toFixed(6),
                    lng: map?.getCenter().lng?.toFixed(6),
                    zoom: map?.getZoom(),
                },
            }}
        >
            <a target="_blank" rel="noreferrer" onClick={closePopover}>
                &#10711; {`${title}`}
            </a>
        </Link>
    );
}
const TableRow = ({ unitat }) => {
    const { object } = unitat;
    const { properties } = unitat;
    const { geometry } = unitat;
    const { coordinates } = geometry;
    const { gps, ioElements } = properties;
    const { map } = MapContext();
    const { setObjects } = ObjectsContext();
    const [account, setaccount] = useState(false);
    const [ampliades, setampliades] = useState(false);

    const handleEditclick = (e) => {
        setaccount(true);
    };

    const handleAmpliadesclick = (e) => {
        setampliades(true);
    };

    const handleChangeChk = ({ target }) => {
        setObjects((prevState) => ({
            error: prevState.error,
            loading: prevState.loading,
            data: prevState.data.map((el) => {
                if (el._id === unitat._id) {
                    return {
                        ...el,
                        object: { ...el.object, visible: target.checked },
                    };
                } else {
                    return el;
                }
            }),
        }));
    };

    const popover = (
        <Popover>
            <Popover.Title as="h3" className="text-center">
                Accions {object?.name}
            </Popover.Title>
            <Popover.Content
                onClick={() => {
                    handleAmpliadesclick();
                    closePopover();
                }}
            >
                Dades Ampliades
            </Popover.Content>
            <Popover.Content>
                <Menu                
                    menuButton={<span>Històric del vehicle ❯</span>}
                    direction="right"
                    align={"center"}
                >
                    <MenuItem>
                        <LinkHistoric
                            unitat={unitat}
                            title="10 minuts"
                            minutes={10}
                        />
                    </MenuItem>
                    <MenuItem>
                        <LinkHistoric
                            unitat={unitat}
                            title="30 minuts"
                            minutes={30}
                        />
                    </MenuItem>
                    <MenuItem>
                        <LinkHistoric
                            unitat={unitat}
                            title="1 hora"
                            minutes={60}
                        />
                    </MenuItem>
                    <MenuItem>
                        <LinkHistoric
                            unitat={unitat}
                            title="2 hores"
                            minutes={120}
                        />
                    </MenuItem>
                    <MenuItem>
                        <LinkHistoric
                            unitat={unitat}
                            title="4 hores"
                            minutes={240}
                        />
                    </MenuItem>
                    <MenuItem>
                        <LinkHistoric
                            unitat={unitat}
                            title="8 hores"
                            minutes={480}
                        />
                    </MenuItem>
                    <MenuItem>
                        <LinkHistoric
                            unitat={unitat}
                            title="12 hores"
                            minutes={720}
                        />
                    </MenuItem>
                    <MenuItem>
                        <LinkHistoric
                            unitat={unitat}
                            title="24 hores"
                            minutes={1440}
                        />
                    </MenuItem>
                </Menu>
            </Popover.Content>
            <Popover.Content onClick={closePopover}>
                <Link
                    href={`https://www.google.com/maps?q&layer=c&cbll=${coordinates[1]},${coordinates[0]}`}
                >
                    <a target="_blank" rel="noreferrer">
                        {"Street View"}
                    </a>
                </Link>
            </Popover.Content>
            <Popover.Content
                onClick={() => {
                    handleEditclick();
                    closePopover();
                }}
            >
                Editar
            </Popover.Content>
        </Popover>
    );

    return (
        <tr key={object?._id}>
            <td className="center">
                {/* <input
                    title="Seguir"
                    className="pointer"
                    type="checkbox"
                    style={{ textAlign: "center" }}
                /> */}
            </td>
            <td className="center">
                <input
                    checked={object?.visible}
                    title="Veure"
                    className="pointer center"
                    type="checkbox"
                    style={{ textAlign: "center" }}
                    onChange={handleChangeChk}
                />
            </td>
            <td className="center">
                <img
                    title={object?.icon?.toUpperCase()}
                    src={`/navLateral/slides/vehicles/${object?.icon}.svg`}
                    alt="Icona de menys"
                    style={{ cursor: "pointer" }}
                    onClick={() =>
                        map.flyTo([coordinates[1], coordinates[0]], 18)
                    }
                />
            </td>
            <td className="center">
                <div style={{ textAlign: "left" }}>{object?.name}</div>
                <div style={{ textAlign: "left" }}>
                    {convertDate(properties?.timestamp)}
                </div>
            </td>
            <td className="center">
                <div style={{ textAlign: "right" }} title="Velocitat">
                    {`${gps?.speed} km/h`}
                </div>
            </td>
            <td className="center">
                <img
                    title={ioElements?.ignition ? "Encès" : "Apagat"}
                    src={
                        ioElements?.ignition
                            ? "/navLateral/slides/engineBlue.svg"
                            : "/navLateral/slides/engine.svg"
                    }
                    alt="Icona de motor"
                    style={{
                        display:
                            properties?.type === "dongle" ? "initial" : "none",
                    }}
                />
            </td>
            <td className="center">
                <img
                    title={
                        ioElements?.gsm_strengh > 0
                            ? `Connectat ${ioElements?.gsm_strengh}`
                            : "Desconnectat"
                    }
                    src={
                        properties?.type === "dongle"
                            ? `/navLateral/slides/signal${iconStrengh(
                                  ioElements?.gsm_strengh
                              )}.svg`
                            : "/navLateral/slides/signal4.svg"
                    }
                    alt="Icona de senyal"
                />
            </td>
            <td className="center pointer">
                <OverlayTrigger
                    placement={
                        window.matchMedia("(max-width: 600px)").matches ||
                        window.matchMedia("(max-height: 600px)").matches
                            ? "left"
                            : "right"
                    }
                    trigger="click"
                    rootClose
                    overlay={popover}
                >
                    <img
                        title="Accions"
                        className="dotImg"
                        src="/navLateral/slides/dots.svg"
                        alt="Icona de menys"
                    />
                </OverlayTrigger>
            </td>
            {!!account && (
                <ModalSettings
                    setshow={setaccount}
                    show={account}
                    title={"Configuració"}
                    showTab={"device"}
                />
            )}
            {!!ampliades && (
                <ModalAmpliada
                    setshow={setampliades}
                    show={ampliades}
                    title={"Dades ampliades"}
                    data={unitat}
                />
            )}
            <style jsx>{`
                :global(.popover-body) {
                    border-bottom: 1px #8a8a8a90 solid;
                    cursor: pointer;
                }
                img {
                    margin: 0px 2px;
                    width: 20px;
                    height: 20px;
                }
                .pointer {
                    cursor: pointer;
                }

                .center {
                    align-self: center;
                    text-align: center;
                }

                td > input {
                    margin-top: 5px;
                }
                td > div {
                    margin-left: 2px;
                }
                .dotImg {
                    margin: 0px;
                    width: 16px;
                    height: 16px;
                }
            `}</style>
        </tr>
    );
};

export default TableRow;
