/* eslint-disable @next/next/no-img-element */
import React, { useState } from "react";
import { colors } from "../../../../../styles/theme";
import Spinner from "../../../../globals/Spinner";
import TablesObjects from "./TablesObjects";
import ObjectsContext from "../../../../../contexts/ObjectsContext";

const VehiclesSlide = () => {
    const [querySearch, setquerySearch] = useState("");

    const { objects } = ObjectsContext();
    const { loading, data, error } = objects;

    const handleSearch = (e) => {
        setquerySearch(e.target.value);
    };

    const separateForType = React.useMemo(() => {
        const map = new Map();
        try {
            data?.map((item) => {
                const key = item?.object.group;
                const collection = map.get(key);
                if (!collection) {
                    map.set(key, [item]);
                } else {
                    collection.push(item);
                }
            });
        } catch (err) {
            console.log(err);
        }
        return map;
    }, [data]);

    return (
        <>
            <div className="groupSearch">
                <input
                    id="search"
                    onChange={handleSearch}
                    placeholder="Cerca"
                    type="search"
                    value={querySearch}
                />
                {/* <div className="iconRefresh" title="Refrescar dades">
                    <img
                        src="navLateral/slides/data-sync.svg"
                        alt="icona sincronització"
                    />
                </div> */}
            </div>
            {loading && !data && (
                <div
                    id="spinner"
                    style={{ visibility: "visible" }}
                    className="spinnerDiv"
                >
                    <Spinner colorspinner={colors.primaryBlue} />
                </div>
            )}

            {error && (
                <h4>
                    Alguna cosa ha fallat, no es poden carregar les dades,
                    recarregui
                </h4>
            )}

            {/* {!loading && data && ( */}
            <TablesObjects data={separateForType} querySearch={querySearch} />
            {/* )} */}

            <style jsx>{`
                .groupSearch {
                    display: flex;
                }
                #search {
                    background-size: 15px 15px;
                    background: #fff url(navLateral/slides/search.svg) 10px
                        center no-repeat;
                    border: 1px solid var(--greyBorder);
                    height: 28px;
                    text-indent: 30px;
                    width: 100%;
                }
                #search:focus {
                    border: 1px solid var(--primaryBlue);
                    outline: none;
                }

                .iconRefresh {
                    background: #8a8a8a50;
                    border-radius: 20px;
                    height: 32px;
                    margin: -1px 0px 0px 8px;
                    padding: 1px 4px;
                    user-select: none;
                    width: 32px;
                }
                .iconRefresh:hover {
                    background: #007bffae;
                    cursor: pointer;
                }

                img {
                    height: 25px;
                    pointer-events: none;
                    user-select: none;
                    width: 25px;
                }
                .spinnerDiv {
                    position: absolute;
                    left: 50%;
                    margin: 0 0 0 -75px;
                    top: 150px;
                }
            `}</style>
        </>
    );
};

export default VehiclesSlide;
