import Image from "next/image";
import { useState } from "react";
import useUserContext from "../../../contexts/UserContext";
import { ModalSettings } from "../../customModals/ModalSettings/ModalSettings";

const TopPanel = ({ margin,data }) => {
    const { user, logout } = useUserContext();
    const [account, setaccount] = useState(false);

    const handleLogout = () => {
        logout();
    };

    const handleclick = () => {
        setaccount(true);
    };

    return (
        <div className="toppanel">
            <div className="nameEntity">
                <span>{data?.displayName}</span>
            </div>
            <div className="accountContainer">
                <div className="account" onClick={handleclick}>
                    <Image
                        src="/navTop/pawn.svg"
                        alt="icono account"
                        width={30}
                        height={30}
                    />
                    <div className="emailEntity">
                        <span>{user?.email}</span>
                    </div>
                </div>
                <div className="logout" onClick={handleLogout}>
                    <Image
                        src="/navTop/logout.svg"
                        alt="icono logout"
                        width={30}
                        height={30}
                    />
                </div>
            </div>
            {!!account && (
                <ModalSettings
                    setshow={setaccount}
                    show={account}
                    title={"Configuració"}
                    text={"Els correus electrònics no són iguals"}
                />
            )}
            <style jsx>{`
                .toppanel {
                    display: grid;
                    grid-template-columns: 30% 40% 30%;
                    grid-template-rows: 3rem;
                    background: var(--background);
                    align-items: center;
                    box-shadow: 0 0.125rem 0.125rem #888;
                    color: var(--title);
                    flex-direction: column;
                    font-style: normal;
                    font-weight: 600;
                    height: 3rem;
                    justify-items: center;
                    position: fixed;
                    top: 0;
                    width: calc(100% - ${margin}rem);
                    z-index: 1;
                }
                .account,
                .logout {
                    margin: 0;
                    display: flex;
                    z-index: 0;
                    padding: 4px;
                    border-radius: 20px;
                    user-select: none;
                }

                .account > img,
                .account > div {
                    pointer-events: none;
                }

                .account:hover,
                .logout:hover {
                    cursor: pointer;
                    background: #007bffae;
                }

                .nameEntity {
                    text-align: center;
                    padding: 0rem 1rem;
                    grid-column: 2;
                }
                .emailEntity {
                    pointer-events: none;
                    margin: 0px 0.5em;
                    align-self: center;
                }
                .accountContainer {
                    text-align: center;
                    padding-right: 1rem;
                    justify-self: end;
                    grid-column: 3;
                    font-size: 0.8rem;
                    display: flex;
                }

                /***********MOBIL*****************/
                @media (max-width: 900px) {
                    .emailEntity,
                    .nameEntity {
                        display: none;
                    }
                }

                /*Landscape & Portrait*/
                @media (max-width: 600px), (max-height: 600px) {
                    .toppanel {
                        z-index: 1;
                        height: 3.75rem;
                        width: calc(100% + 0rem);
                        padding-left: 4rem;
                        grid-template-rows: 3.75rem;
                    }
                }

                @media (max-width: 600px) {
                    .toppanel {
                        display: grid;
                        grid-template-columns: 70% 30%;
                        grid-template-rows: 3.75rem;
                        height: 3.75rem;
                        z-index: 1;
                    }

                    .accountContainer {
                        grid-column: 2;
                    }
                }
            `}</style>
        </div>
    );
};

export default TopPanel;
