import { Button } from "../globals/Button";
import { FormCheck } from "react-bootstrap";
import { General } from "../formulari/General";
import { ModalClose } from "../globals/ModalClose";
import { Password } from "../formulari/Password";
import { useEffect, useState } from "react";
import Form from "react-bootstrap/Form";
import InputGroup from "react-bootstrap/InputGroup";
import styles from "../../styles/signUp.module.css";

export const AllDataCheck = ({ handleInputChange, formValues }) => {
    //Modals
    const [show, setshow] = useState(false);
    const [modalRegister, setmodalRegister] = useState(false);
    const [modalError, setmodalError] = useState(false);

    const [formValidated, setformValidated] = useState(false);

    const { /*email,*/ displayName, password, cPassword } = formValues;

    const changeInput = (e) => {
        handleInputChange(e);
        //Si el formulari es vàlid
        if (formValidated) {
            setformValidated(false);
        }
    };

    const submitEmail = async (e) => {
        const form = e.currentTarget;
        e.preventDefault();
        e.stopPropagation();

        if (form.checkValidity() === true) {
            if (password === cPassword) {
                //Envia dades a la API per crear nou compte
                //API fa tota la logica a Mongo. Retorna OK.
                fetch("api/auth/createUser", {
                    method: "POST",
                    headers: {
                        Accept: "application/json",
                        "Content-Type": "application/json",
                    },
                    body: JSON.stringify(formValues),
                })
                    .then((resp) => resp.json())
                    .then((data) => {
                        console.log(data);
                        //User created
                        data?.user && setmodalRegister(true);
                        //User not created
                        !data?.user && setmodalError(true);
                    })
                    .catch((err) => {
                        console.error(err);
                        setmodalError(true);
                    });
            } else {
                setshow(true);
            }
        } else {
            setformValidated(true);
        }
    };

    /*Focus first email if user exists*/
    useEffect(() => {
        document.getElementById("password").focus();
    }, []);

    return (
        <div className={styles.checkEmailContainer}>
            <div style={{ maxWidth: "600px" }}>
                <div className={styles.registerPart}>
                    Registre
                    <div className={styles.triangleRight} />
                    <span>
                        {" "}
                        Part <strong>2</strong> de 2
                    </span>
                </div>

                <Form
                    noValidate
                    validated={formValidated}
                    onSubmit={submitEmail}
                >
                    <div className={styles.card}>
                        <Password
                            changeInput={changeInput}
                            id={"password"}
                            name={"password"}
                            value={password}
                        />
                        <Password
                            changeInput={changeInput}
                            id={"cPassword"}
                            label={"Confirmar paraula de pas"}
                            name={"cPassword"}
                            value={cPassword}
                        />
                    </div>
                    <div className={styles.card}>
                        <General
                            changeInput={changeInput}
                            id={"displayName"}
                            label={"Nom de l'entitat"}
                            name={"displayName"}
                            value={displayName}
                            type={"text"}
                        />
                    </div>
                    <div className={styles.card}>
                        <InputGroup>
                            <FormCheck>
                                <FormCheck.Input required />
                                <FormCheck.Label>
                                    Estic d&lsquo;acord amb els Termes i
                                    condicions d&lsquo;ús legal i la política de
                                    privacitat.
                                </FormCheck.Label>
                                <Form.Control.Feedback type="invalid">
                                    És obligatori llegir i acceptar els termes i
                                    condicions.
                                </Form.Control.Feedback>
                            </FormCheck>
                        </InputGroup>
                    </div>
                    <div className={styles.card}>
                        <Form.Row className="mt-1 d-flex justify-content-center">
                            <Button type="submit">Completar registre</Button>
                        </Form.Row>
                    </div>
                </Form>
            </div>
            <ModalClose
                centered
                setshow={setshow}
                show={show}
                title={"Error de validació"}
                text={"Les paraules de pas no són igual"}
            />
            <ModalClose
                centered
                setshow={setmodalRegister}
                show={modalRegister}
                title={"Registre realitzat correctament"}
                text={
                    "S'ha enviat un enllaç al correu elèctronic per verificar la teva adreça, segueixi les intruccions enviades per finalitzar el registre"
                }
                route="/"
            />
            <ModalClose
                centered
                setshow={setmodalError}
                show={modalError}
                title={"Alguna cosa a fallat!"}
                text={
                    "No s'ha pogut crear el nou usuari, intenta provar de nou o contacti amb l'administrador"
                }
                route="/"
            />
        </div>
    );
};
