/**
 * Component que es carrega a la pagina de signUp i que
 * farà totes les comprovacions del email a l'hora de registrar-se
 * si tot es correcte retorna setemailCheck = true i fa el salt a la
 * segona pagina per acabar d'introduïr les dades de registre.
 */

import { Button } from "../../components/globals/Button";
import Form from "react-bootstrap/Form";
import Link from "next/link";
import styles from "../../styles/signUp.module.css";
import { useEffect, useState } from "react";
import { ModalClose } from "../globals/ModalClose";
import { UserName } from "../formulari/UserName";

export const EmailCheck = ({
    handleInputChange,
    formValues,
    setemailCheck,
    reset,
}) => {
    const [show, setshow] = useState(false);

    const [showUserExist, setshowUserExist] = useState(false);

    const [formValidated, setformValidated] = useState(false);

    const { email, cEmail } = formValues;

    const changeInput = (e) => {
        handleInputChange(e);
        if (formValidated) {
            setformValidated(false);
        }
    };

    const submitEmail = async (e) => {
        const form = e.currentTarget;
        e.preventDefault();
        e.stopPropagation();

        if (form.checkValidity() === true) {
            if (email === cEmail) {
                const url = `/api/auth/checkUserByEmail`;

                fetch(url, {
                    method: "post",
                    headers: {
                        Accept: "application/json, text/plain, */*",
                        "Content-Type": "application/json",
                    },
                    body: JSON.stringify({ email: email }),
                })
                    .then((resp) => resp.json())
                    .then(({ doesExist }) => {
                        if (doesExist) {
                            setshowUserExist(true);
                            reset();
                        } else {
                            setemailCheck(true);
                        }
                    })
                    .catch((error) => {
                        console.error(error);
                    });
            } else {
                setshow(true);
            }
        } else {
            setformValidated(true);
        }
    };

    /*Focus first email if user exists*/
    useEffect(() => {
        document.getElementById("email").focus();
    }, [showUserExist]);

    return (
        <div className={styles.checkEmailContainer}>
            <div style={{ maxWidth: "600px" }}>
                <div className={styles.registerPart}>
                    Registre
                    <div className={styles.triangleRight} />
                    <span>
                        {" "}
                        Part <strong>1</strong> de 2
                    </span>
                </div>
                <div className={styles.card}>
                    <Form
                        noValidate
                        validated={formValidated}
                        onSubmit={submitEmail}
                    >
                        <UserName
                            changeInput={changeInput}
                            email={email}
                            id={"email"}
                            name={"email"}
                        />
                        <UserName
                            changeInput={changeInput}
                            email={cEmail}
                            id={"cEmail"}
                            label={"Confirmi la direcció de correu"}
                            name={"cEmail"}
                        />

                        <Form.Row
                            className="mt-3 col-md-12"
                            style={{ textAlign: "justify" }}
                        >
                            <Form.Text
                                style={{
                                    fontSize: "0.9em",
                                }}
                            >
                                La vostra adreça de correu electrònic serà el
                                vostre nom d&lsquo;usuari. L&lsquo;utilitzaràs per iniciar
                                la sessió. La nostra 
                                {"  "}
                                <Link href="/privacyPolicy">
                                    <a> Política de privadesa</a>
                                </Link>
                                {"  "}
                                estableix com es gestiona la vostra informació
                                personal i els vostres drets d’oposició a que la
                                vostra informació personal s&lsquo;utilitzi amb
                                finalitats comercials. En aquest lloc únicament
                                utilitzarem la vostra informació personal per
                                proporcionar-vos aquest servei.
                            </Form.Text>
                        </Form.Row>
                        <Form.Row className="mt-4 d-flex justify-content-center">
                            <Button type="submit">Continuar</Button>
                        </Form.Row>
                    </Form>
                </div>
            </div>

            {/* components modals que apareixen en cas d'errors al email */}

            <ModalClose                
                centered
                setshow={setshow}
                show={show}
                title={"Error de validació"}
                text={"Els correus electrònics no són iguals"}
            />
            <ModalClose
                
                centered
                setshow={setshowUserExist}
                show={showUserExist}
                title={"Error de validació"}
                text={
                    "El correu electrònic introduït ja correspon a un usuari registrat"
                }
            />
        </div>
    );
};
