/**
 *
 *
 */

import Account from "../../../models/Account";
import mongoose from "mongoose";
import { connectToDatabase } from "../../../utils/db/mongodb";
import { sendJsonResponse } from "../../../utils/lib/responseServer";
import { withAuth } from "../../../utils/lib/middlewares";
import Service from "../../../models/Service";
import { pusher } from "../pusher";

const getOffset = (offset) => {
    let retorno = offset;
    if (retorno < 0) {
        retorno *= -1;
        retorno /= 60;
        retorno = `+0${retorno}:00`;
    } else {
        retorno %= 60;
        retorno = `-0${retorno}:00`;
    }

    return retorno;
};

const services = async (req, res) => {
    //Connect mongoose to DB
    await connectToDatabase();
    const method = req.method;
    const { data: pdata } = req.body || false;
    const user_id = req?.user_id;
    const token = req?.headers?.token;
    //Cercar a la BBDD el compte que fa l'acció a partir del id l'usuari
    const accountSet = await Account.findById(user_id).lean();
    //Definim les possibles funcions
    const actions = {
        GET: () => isGET(),
        POST: () => isPOST(),
        PUT: () => isPUT(),
        DELETE: () => isDELETE(),
    };

    ////Implementació de les funcions////
    const isGET = async () => {
        try {
            let {
                query: { dInici, dFinal },
            } = req;

            let monthless = new Date();
            let offset = getOffset(monthless.getTimezoneOffset());
            monthless.setDate(monthless.getDate() - 1); //Ultim dia 24h

            if (!dInici || !dFinal) {
                dInici = pdata?.dataInit ?? monthless;
                dFinal = pdata?.dataFinish ?? Date.now();
            } else {
                dInici += offset;
                dFinal += offset;
            }

            const services = await Service.find({
                idEntity: accountSet?.idEntity,
                dateInput: { $gte: dInici, $lte: dFinal },
            });
            sendJsonResponse(res, 200, { accion: true, data: services });
        } catch (error) {
            console.log(error);
            sendJsonResponse(res, 500, {
                accion: false,
                err: error,
            });
        }
    };

    const isPUT = async () => {
        try {
            let dateAssign = () => {
                if (!!pdata?.dateAssign) {
                    return pdata.dateAssign;
                } else if (pdata?.objects?.length > 0) {
                    return Date.now();
                } else {
                    return undefined;
                }
            };

            let status = () => {
                if (pdata?.status === "closed") {
                    return "closed";
                } else {
                    return pdata?.objects?.length > 0 ? "traspas" : "open";
                }
            };

            let finish = () => {
                if (!!pdata?.dataFinish) {
                    return pdata?.dataFinish;
                } else {
                    if (pdata?.status === "closed") {
                        return Date.now();
                    }
                }
            };

            const updateData = {
                title: pdata?.title ?? undefined,
                dateAssign: dateAssign(),
                dateFinish: finish(),
                geometry: {
                    coordinates: [pdata?.lon, pdata?.lat], //Centre Catalunya
                },
                direction: pdata?.direction ?? "",
                service: pdata?.service ?? "",
                result: pdata?.result ?? "",
                status: status(),
                // objects: pdata?.objects ?? [],
            };

            if (!pdata?._id) {
                throw new Error("No es pot updatar un servei sense ID");
            }

            const filter = mongoose.Types.ObjectId(pdata?._id);
            let service = await Service.findByIdAndUpdate(filter, updateData, {
                new: true,
            });
            if (service) {
                if (pdata?.objects?.length) {
                    service.objects = pdata?.objects;
                } else {
                    service.objects = [];
                }
                service = await service.save();

                const idEntity = accountSet?.idEntity ?? "";
                const response = await pusher.trigger(
                    `${idEntity}`,
                    `services-${idEntity}`,
                    {
                        service,
                        emmit: token,
                    }
                );

                sendJsonResponse(res, 200, {
                    accion: true,
                    data: service,
                });
            } else {
                sendJsonResponse(res, 500, {
                    accion: false,
                    err: "No s'ha pogut crear gravar el servei",
                });
            }
        } catch (error) {
            console.log(error);
            return sendJsonResponse(res, 500, {
                accion: false,
                err: error,
            });
        }
    };

    const isPOST = async () => {
        try {
            const service = await new Service({
                idEntity: accountSet?.idEntity,
                idUserInit: accountSet?._id,
                title: pdata?.title,
                dateAssign: pdata?.objects?.length > 0 ? Date.now() : undefined,
                geometry: {
                    coordinates: [pdata?.lon, pdata?.lat], //Centre Catalunya
                },
                direction: pdata?.direction ?? "",
                service: pdata?.service ?? "",
                result: pdata?.result ?? "",
                status: pdata?.objects?.length > 0 ? "traspas" : "open",
                objects: pdata?.objects ?? [],
            }).save();

            if (service) {
                const idEntity = accountSet?.idEntity ?? "";
                const response = await pusher.trigger(
                    `${idEntity}`,
                    `services-${idEntity}`,
                    {
                        service,
                        emmit: token,
                    }
                );

                //Gravar en la BBDD
                sendJsonResponse(res, 200, {
                    accion: true,
                    data: service,
                });
            } else {
                sendJsonResponse(res, 500, {
                    accion: false,
                    err: "No s'ha pogut crear gravar el servei",
                });
            }
        } catch (error) {
            console.log(error);
            return sendJsonResponse(res, 500, {
                accion: false,
                err: error,
            });
        }
    };

    const isDELETE = async () => {
        return sendJsonResponse(res, 405, {
            accion: false,
            err: "Metode no permès",
        });
    };

    return actions[method]();
};
export default withAuth(services);
