/**
 * Crea un usuari nou a Firebase i a Mongo
 * Si falla algun dels dos fa RollBack i borra tot
 * També envia un correu de verificació a l'adreça indicada.
 * L'enviament del correu no influeix, es fa en asincron, si falla
 * aquest enviament es grava l'usuari igualment, es pot demanar tornar a
 * enviar si s'entra sense estar verificat.
 */

import Account from "../../../models/Account";
import Entity from "../../../models/Entity";
import { firebaseAdmin } from "../../../utils/auth/firebaseAdmin";
import { connectToDatabase } from "../../../utils/db/mongodb";
import { returnUserByToken } from "../functions";
import mongoose from "mongoose";
import { sendJsonResponse } from "../../../utils/lib/responseServer";
import { withAuth, withRoles } from "../../../utils/lib/middlewares";
import { HostAddress } from "mongodb";

const entityData = async (req, res) => {
    //Connect mongoose to DB
    await connectToDatabase();
    const method = req.method;
    const { data: pdata } = req.body || false;
    const user_id = req?.user_id;
    //Cercar a la BBDD el compte que fa l'acció a partir del id l'usuari
    const accountSet = await Account.findById(user_id).lean();

    //Definim les possibles funcions
    const actions = {
        GET: () => isGET(),
        POST: () => isPUT(),
        PUT: () => isPUT(),
        DELETE: () => isDELETE(),
    };

    ////Implementació de les funcions////
    const isGET = async () => {
        try {
            //Retorna la entitat a la qual pertany l'usuari
            const entity = await Entity.findById(
                mongoose.Types.ObjectId(accountSet.idEntity)
            ).lean();
            entity
                ? sendJsonResponse(res, 200, { accion: true, data: entity })
                : sendJsonResponse(res, 500, {
                      accion: false,
                      err: "No es poden mostrar les dades de l'entitat",
                  });
        } catch (error) {
            sendJsonResponse(res, 500, {
                accion: false,
                err: error,
            });
        }
    };

    //UPDATE
    const isPUT = async () => {
        try {
            //Fields to update
            let updateData = {
                displayName: pdata.displayName,
                address: pdata.address,
                postCode: pdata.postCode,
                city: pdata.city,
                country: pdata.country,
                phone: pdata.phone,
            };

            //Controlar k sigui admin
            if (accountSet?.admin) {
                //cerca i actualitza la entitat amb les dades passades
                const filter = mongoose.Types.ObjectId(accountSet.idEntity);
                const updateAccount = await Entity.findByIdAndUpdate(
                    filter,
                    updateData,
                    {
                        new: true,
                    }
                );
                !!updateAccount
                    ? sendJsonResponse(res, 200, {
                          accion: true,
                          data: updateAccount,
                      })
                    : sendJsonResponse(res, 500, {
                          accion: false,
                          err: "No s'ha pogut actualitzar la entitat",
                      });
            } else {
                sendJsonResponse(res, 403, {
                    accion: false,
                    err: "No tens permís per a realitzar aquesta acció",
                });
            }
        } catch (error) {
            sendJsonResponse(res, 500, {
                accion: false,
                err: error,
            });
        }
    };

    const isDELETE = async () => {
        //Recuperar usuari Altas a partir del token
        //Fer servir lean en comptes de exec, molt més lleuger
        // const account = await Account.findOne({
        //     idAccount: user.user_id,
        // }).lean();

        // //Mirar si es Main o Sub
        // if (account.main) {
        //     //Si es principal

        //     account?.subAccounts.length > 0 &&
        //         account.subAccounts.map((idSubAccount) => {
        //             //Borrar comptes secundaris de Atlas
        //             Account.findOneAndRemove(
        //                 { idAccount: idSubAccount },
        //                 (err, doc) => {
        //                     console.log(err);
        //                     console.log(doc);
        //                 }
        //             ).exec();
        //             //Borrar comptes secundaris de FireBase
        //             firebaseAdmin.auth().deleteUser(idSubAccount);
        //         });
        //     //Borrar compte principal
        //     Account.findOneAndRemove({ idAccount: idAccount }, (err, doc) => {
        //         console.log(err);
        //         console.log(doc);
        //     }).exec();
        // } else {
        //     //Si és secundaria
        //     //Borrar compte de Atlas
        //     firebaseAdmin.auth().deleteUser(idAccount);
        //     //Borrar dels subAccounts del principal

        // }

        res.status(200).send({ accion: false });

        //Pedir otra vez contraseña antes de eliminar
        // firebase
        //     .auth()
        //     .signInWithEmailAndPassword("rulomo@hotmail.com", "Recall19*")
        //     .then(({ user }) => {
        //         // Password is OK
        //         //Eliminar de Firebase
        //         user.delete()
        //             .then(() => {
        //                 //Eliminar la cuenta normal de Mongo
        //                 Account.findOneAndRemove(
        //                     { idAccount: user.uid },
        //                     (err, doc) => {
        //                         //Sino és main eliminar de subcuenta en main

        //                     }
        //                 );
        //             })
        //             .catch((error) => {
        //                 toast.info(
        //                     `No s'ha pogut suprimir el compte ${
        //                         user?.email || ""
        //                     }`
        //                 );
        //             });
        //     })
        //     .catch((error) => {
        //         const errorCode = error.code;
        //         const errorMessage = error.message;
        //         console.log("ERRORRRRRRRRRRRRRRRRRRRR");
        //     });
    };

    return actions[method]();
};

export default withAuth(entityData);
