import { connectToDatabase } from "../../../utils/db/mongodb";
import Account from "../../../models/Account";
import Object from "../../../models/Object";
import Unitat from "../../../models/Unitat";
import mongoose from "mongoose";
import { withAuth } from "../../../utils/lib/middlewares";

const mainData = async (req, res) => {
    //Connect mongoose to DB
    await connectToDatabase();
    const method = req.method;
    const { data: pdata } = req.body || false;
    const user_id = req?.user_id;
    //Cercar a la BBDD el compte que fa l'acció a partir del id l'usuari
    const account = await Account.findById(user_id).lean();

    try {
        //Extreu tots els imeis dels objectes que pertanyen a la mateixa entitat
        const imeis = await Object.distinct("IMEI", {
            idEntity: mongoose.Types.ObjectId(account.idEntity),
        }).lean();

        //Tornar les ultimes marcades de cada unitat i incorporar un objecte amb el
        //"objecte ("GPS,movil,....") al que pertany.
        const unitats = await Unitat.aggregate([
            { $match: { "properties.imei": { $in: imeis } } },
            {
                $group: {
                    _id: "$properties.imei",
                    timestamp: {
                        $last: "$properties.timestamp",
                    },
                    detail: { $last: "$$ROOT" },
                },
            },
            {
                $lookup: {
                    from: "objects",
                    as: "object",
                    let: { imei_unitat: "$detail.properties.imei" },
                    pipeline: [
                        {
                            $match: {
                                $expr: {
                                    $and: [
                                        {
                                            $eq: ["$IMEI", "$$imei_unitat"],
                                        },
                                    ],
                                },
                            },
                        },
                    ],
                },
            },
            {
                $replaceRoot: {
                    newRoot: {
                        $mergeObjects: [{ object: "$object" }, "$detail"],
                    },
                },
            },
        ]).exec();

        //Tornava un array dintre d'objecte, ho poso com un objecte directe
        unitats.map((item) => {
            item.object = item.object[0];
        });

        //retornar les dades.
        res.status(200).json({ data: unitats });
    } catch (error) {
        res.status(500).send({
            data: [],
            error: "Alguna cosa ha fallat, no es poden mostrar les dades demanades",
        });
    }
};

export default withAuth(mainData);
