/**
 *
 *
 */

import Account from "../../../models/Account";
import Object from "../../../models/Object";
import { connectToDatabase } from "../../../utils/db/mongodb";
import { sendJsonResponse } from "../../../utils/lib/responseServer";
import { withAuth, withRoles } from "../../../utils/lib/middlewares";

const objectsData = async (req, res) => {
    //Connect mongoose to DB
    await connectToDatabase();
    const method = req.method;
    const { data: pdata } = req.body || false;
    const user_id = req?.user_id;
    //Cercar a la BBDD el compte que fa l'acció a partir del id l'usuari
    const accountSet = await Account.findById(user_id).lean();

    //Definim les possibles funcions
    const actions = {
        GET: () => isGET(),
        POST: () => isPOST(),
        PUT: () => isPUT(),
        DELETE: () => isDELETE(),
    };

    ////Implementació de les funcions////
    const isGET = async () => {
        try {
            //Get objectes
            const object = await Object.find({
                idEntity: accountSet.idEntity,
            });

            !!object
                ? sendJsonResponse(res, 200, {
                      accion: true,
                      data: object,
                  })
                : sendJsonResponse(res, 500, {
                      accion: false,
                      err: "No es poden obtenir les dades sol·licitades",
                  });
        } catch (error) {
            sendJsonResponse(res, 500, {
                accion: false,
                err: "no es poden obtenir les dades sol·licitades",
            });
        }
    };

    const isPUT = async () => {
        try {
            //Cercar a la BBDD l'objecte sobre el qual es fará l'acció
            const objectGet = await Object.findById(pdata._id).lean();

            //Fields to update
            let updateData = {
                IMEI: pdata?.IMEI,
                marca: pdata?.marca,
                model: pdata?.model,
                name: pdata?.name,
                VIN: pdata?.VIN,
                plateNumber: pdata?.plateNumber,
                group: pdata?.group,
                icon: pdata?.icon,
                GPSdevice: pdata?.GPSdevice,
                visible: pdata?.visible,
                active: pdata?.active,
            };

            //Si no pertanyen a la mateixa entitat no es pot modificar
            if (
                accountSet.idEntity.toString() === objectGet.idEntity.toString()
            ) {
                //Update object in BBDD
                Object.findByIdAndUpdate(
                    objectGet._id,
                    updateData,
                    {
                        new: true,
                    },
                    (err, model) => {
                        !err
                            ? sendJsonResponse(res, 200, {
                                  accion: true,
                                  data: model,
                              })
                            : sendJsonResponse(res, 500, {
                                  accion: false,
                                  err: err,
                              });
                    }
                );
            } else {
                sendJsonResponse(res, 403, {
                    accion: false,
                    err: "No autorització per fer canvis a l'objecte",
                });
            }
        } catch (error) {
            return sendJsonResponse(res, 500, {
                accion: false,
                err: error,
            });
        }
    };

    const isPOST = async () => {
        try {
            //Cercar si hi ha algun objecte amb el mateix IMEI
            let imeiFound = await Object.findOne({ IMEI: pdata.IMEI });

            //Si ja existeix imei no es pot gravar
            if (!imeiFound) {
                //Assignar la entitat a l'objecte
                pdata.idEntity = accountSet.idEntity;
                const object = new Object(pdata);
                object.save().then((savedOject) => {
                    //Si el crea correctament
                    object === savedOject
                        ? sendJsonResponse(res, 200, {
                              accion: true,
                              data: savedOject,
                          })
                        : sendJsonResponse(res, 500, {
                              accion: false,
                              err: "No s'ha pogut crear l'objecte",
                          });
                });
            } else {
                sendJsonResponse(res, 403, {
                    accion: false,
                    err: "IMEI ja existeix",
                });
            }
        } catch (error) {
            return sendJsonResponse(res, 500, {
                accion: false,
                err: error,
            });
        }
    };

    const isDELETE = async () => {
        try {
            //Cercar a la BBDD l'objecte sobre el qual es fará l'acció
            const objectGet = await Object.findById(pdata._id).lean();

            //Si no pertanyen a la mateixa entitat no es pot borrar
            if (
                accountSet.idEntity.toString() === objectGet.idEntity.toString()
            ) {
                //Borrar l'objecte
                const object = await Object.findByIdAndRemove(objectGet._id);
                !!object
                    ? sendJsonResponse(res, 200, { accion: true, data: object })
                    : sendJsonResponse(res, 500, {
                          accion: false,
                          err: "No s'ha trobat l'objecte a la base de dades",
                      });
            } else {
                sendJsonResponse(res, 403, {
                    accion: false,
                    err: "No es pot borrar l'objecte, no tens privilegis",
                });
            }
        } catch (error) {
            return sendJsonResponse(res, 500, {
                accion: false,
                err: error,
            });
        }
    };

    return actions[method]();
};
export default withAuth(withRoles(objectsData, "admin"));
