/**
 * Crea un usuari nou a Firebase i a Mongo
 * Si falla algun dels dos fa RollBack i borra tot
 * També envia un correu de verificació a l'adreça indicada.
 * L'enviament del correu no influeix, es fa en asincron, si falla
 * aquest enviament es grava l'usuari igualment, es pot demanar tornar a
 * enviar si s'entra sense estar verificat.
 */

import Account, {
    createAccountInFirebase,
    createAccountInMongoDB,
    createLinkVerificationEmail,
    rollBackAccountFirebase,
    sendVerificationEmail,
} from "../../../models/Account";

import { connectToDatabase } from "../../../utils/db/mongodb";
import { withAuth, withRoles } from "../../../utils/lib/middlewares";
import { sendJsonResponse } from "../../../utils/lib/responseServer";

const { EMAIL_USER, EMAIL_PASS } = process.env;

const subaccountsData = async (req, res) => {
    //Connect mongoose to DB
    await connectToDatabase();
    const method = req.method;
    const { data: pdata } = req.body || false;
    const user_id = req?.user_id;
    //Cercar a la BBDD el compte que fa l'acció a partir del id l'usuari
    const accountSet = await Account.findById(user_id).lean();

    //Definim les possibles funcions
    const actions = {
        GET: () => isGET(),
        POST: () => isPOST(),
        PUT: () => isPUT(),
        DELETE: () => isDELETE(),
    };

    ////Implementació de les funcions////
    const isGET = async () => {
        //Get subaccounts
        try {
            const accounts = await Account.find({
                idEntity: accountSet.idEntity,
            });

            !!accounts
                ? sendJsonResponse(res, 200, {
                      accion: true,
                      data: accounts,
                  })
                : sendJsonResponse(res, 500, {
                      accion: false,
                      err: "No es poden obtenir les dades sol·licitades",
                  });

        } catch (error) {
            sendJsonResponse(res, 500, {
                accion: false,
                err: "no es poden obtenir les dades sol·licitades",
            });
        }
    };

    const isPUT = async () => {
        try {
            //Cercar a la BBDD el compte sobre el qual es fará l'acció
            const accountGet = await Account.findById(pdata._id).lean();

            let updateData = {
                displayName: pdata?.displayName,
                admin: pdata?.admin,
                active: pdata?.active,
            };

            //Si no pertanyen a la mateixa entitat no es pot fer res
            if (
                accountSet.idEntity.toString() ===
                accountGet.idEntity.toString()
            ) {
                //Si ets tu mateix no pots eliminar-te de administrador o d'actiu.
                if (
                    accountSet._id.toString() == accountGet._id.toString() &&
                    (accountSet?.admin != updateData?.admin ||
                        accountSet?.active != updateData?.active)
                ) {
                    return sendJsonResponse(res, 403, {
                        accion: false,
                        err: "No es pot desactivar un compte a si mateix",
                    });
                }

                //Si s'intenta canviar un main no és pot si no es un mateix
                if (
                    accountGet.main &&
                    accountSet._id.toString() != accountGet._id.toString()
                ) {
                    return sendJsonResponse(res, 403, {
                        accion: false,
                        err: "No autorització per fer canvis al compte",
                    });
                }

                //Update account in BBDD
                Account.findByIdAndUpdate(
                    accountGet._id,
                    updateData,
                    {
                        new: true,
                    },
                    (err, account) => {
                        !err
                            ? sendJsonResponse(res, 200, {
                                  accion: true,
                                  data: account,
                              })
                            : sendJsonResponse(res, 500, {
                                  accion: false,
                                  err: err,
                              });
                    }
                );
            } else {
                sendJsonResponse(res, 403, {
                    accion: false,
                    err: "No autorització per fer canvis al compte",
                });
            }
        } catch (error) {
            sendJsonResponse(res, 500, {
                accion: false,
                err: error,
            });
        }
    };

    const isPOST = async () => {
        try {
            //Cercar a la BBDD el compte sobre el qual es fará l'acció
            const accountGet = await Account.findOne({
                email: pdata.email,
            }).lean();
            //Si ja esta el correu ja existeix no es pot tornar a utilitzar
            if (!accountGet) {
                //Crear compte a Firebase
                const userFirebase = await createAccountInFirebase({
                    email: pdata.email,
                    password: "T3MP0R4L",
                    displayName: pdata.displayName,
                });
                //Crear compte a MongoDB
                const userMongo = await createAccountInMongoDB({
                    uid: userFirebase.uid,
                    email: pdata.email,
                    admin: pdata.admin,
                    idEntity: accountSet.idEntity,
                    displayName: pdata.displayName,
                });

                if (userMongo) {
                    //Send Email
                    //Create link verify email
                    createLinkVerificationEmail(userFirebase)
                        //Send Verify email
                        .then((link) =>
                            sendVerificationEmail(
                                userFirebase,
                                link,
                                EMAIL_USER,
                                EMAIL_PASS
                            )
                        )
                        .catch((err) => {
                            console.log(err);
                            console.log(
                                "El link per verificar l'email no s'ha pogut generar"
                            );
                        });
                    //Si Mongo true User final OK
                    sendJsonResponse(res, 200, {
                        accion: true,
                        data: userMongo,
                    });
                    console.log(`user created: ${userMongo._id}`);
                } else {
                    //Si hi ha usuari de Firebase que el borri i doni error
                    userFirebase && rollBackAccountFirebase(userFirebase);
                    sendJsonResponse(res, 500, {
                        accion: false,
                        err: "No s'ha pogut crear el compte",
                    });
                }
            } else {
                sendJsonResponse(res, 403, {
                    accion: false,
                    err: "EMAIL ja existeix",
                });
            }
        } catch (error) {
            sendJsonResponse(res, 500, {
                accion: false,
                err: error,
            });
        }
    };

    const isDELETE = async () => {
        try {
            //Cercar a la BBDD el compte sobre el qual es fará l'acció
            const accountGet = await Account.findById(pdata._id).lean();
            //Si no pertanyen a la mateixa entitat no es pot fer res
            if (
                accountSet.idEntity.toString() ===
                accountGet.idEntity.toString()
            ) {
                //Si ets tu mateix no pots eliminar-te a tu mateix.
                if (accountSet._id.toString() == accountGet._id.toString()) {
                    return sendJsonResponse(res, 403, {
                        accion: false,
                        err: "No es pot eliminar un compte a si mateix",
                    });
                }

                //Si s'intenta eliminar un main no és pot
                if (accountGet.main) {
                    return sendJsonResponse(res, 403, {
                        accion: false,
                        err: "No autorització per fer canvis al compte",
                    });
                }
                const account = await Account.findByIdAndRemove(accountGet._id);
                account && rollBackAccountFirebase({ uid: account._id });
                sendJsonResponse(res, 200, { accion: true, data: account });
            } else {
                sendJsonResponse(res, 403, {
                    accion: false,
                    err: "No es pot borrar el compte, no tens privilegis",
                });
            }
        } catch (error) {
            sendJsonResponse(res, 500, {
                accion: false,
                err: error,
            });
        }
    };

    //Retornem el document executant l'acció que li toca si s'identifica OK
    return actions[method]();
};

export default withAuth(withRoles(subaccountsData, "admin"));
