import { useRouter } from "next/router";
import React, { useEffect } from "react";
import useUserContext from "../contexts/UserContext";
import { colors } from "../styles/theme";
import Spinner from "../components/globals/Spinner";
import Main from "../components/main/Main";
import MainEmailNotVerified from "../components/main/MainEmailNotVerified";
import { ObjectsContextProvider } from "../contexts/ObjectsContext";
import { MapContextProvider } from "../contexts/MapContext";
import { ServeisContextProvider } from "../contexts/ServeisContext";
import { SlideMenuContextProvider } from "../contexts/SlideMenuContext";
import { AccountContextProvider } from "../contexts/AccountContext";
import { ZonesContextProvider } from "../contexts/ZonesContext";

export default function AppMain() {
    const router = useRouter();
    const { user } = useUserContext();

    //Dintre d'un userEffect pq sino el router no esta carregat al principi
    useEffect(() => {
        !user && router.push("/");
    });

    if (user) {
        return (
           
                <SlideMenuContextProvider>
                    <AccountContextProvider>
                        <MapContextProvider>
                            <ObjectsContextProvider>
                                <ServeisContextProvider>
                                    <ZonesContextProvider>
                                        {user?.emailVerified && <Main />}
                                        {!user?.emailVerified && (
                                            <MainEmailNotVerified />
                                        )}
                                    </ZonesContextProvider>
                                </ServeisContextProvider>
                            </ObjectsContextProvider>
                        </MapContextProvider>
                    </AccountContextProvider>
                </SlideMenuContextProvider>
            
        );
    } else {
        return (
            <div
                id="spinner"
                style={{
                    position: "absolute",
                    width: "100vw",
                    display: "flex",
                    flexDirection: "column",
                    top: "40%",
                }}
            >
                <Spinner colorspinner={colors.primaryBlue} />
            </div>
        );
    }
}
