/**
 * Pagina que per a reestablir la paraula de pas 
 * d'un usuari.
 */

import { Button } from "../components/globals/Button";
import React, { useState } from "react";
import Form from "react-bootstrap/Form";
import styles from "../styles/signUp.module.css";
import { useForm } from "../Hooks/useForm";
import { SignUpHeader } from "../components/register/SignUpHeader";
import { ModalClose } from "../components/globals/ModalClose";
import { UserName } from "../components/formulari/UserName";
import firebase from "firebase/app";
import "firebase/auth";

export default function ForgotPasswrod() {
    const changeInput = (e) => {
        handleInputChange(e);
        if (formValidated) {
            setformValidated(false);
        }
    };

    /**Envia al correu especificat un enllaç per reestablir la
     * paraula de pas. Si l'usuari existeix o no sempre informa que 
     * s'ha enviat el correu. Així s'evita que es pugui saber quins 
     * correus estan registrats per un atacant. 
     */
    const handleSubmit = (e) => {
        const form = e.currentTarget;
        e.preventDefault();
        e.stopPropagation();

        if (form.checkValidity() === true) {
            const url = `/api/auth/checkUserByEmail?email=${email}`;

            fetch(url)
                .then((resp) => resp.json())
                .then(({ doesExist }) => {
                    //Si existeix que envii el correu
                    if (doesExist) {
                        firebase
                            .auth()
                            .sendPasswordResetEmail(email)
                            .then(setshow(true))
                            .catch((error) => {
                                console.error(error);
                            });
                    } else {
                        setshowUserNotFound(true);
                    }
                })
                .catch((error) => {
                    console.error(error);
                });
        } else {
            setformValidated(true);
        }
    };

    const initialForm = {
        email: "",
    };

    const [formValues, handleInputChange /*,reset*/] = useForm(initialForm);

    const { email } = formValues;

    const [show, setshow] = useState(false);

    const [showUserNotFound, setshowUserNotFound] = useState(false);

    const [formValidated, setformValidated] = useState(false);

    return (
        <>
            <SignUpHeader />
            <div className={styles.checkEmailContainer}>
                <div className="mt-4" style={{ maxWidth: "600px" }}>
                    <div className={styles.card}>
                        <h2 className={styles.h2}>
                            Recupera la paraula de pas
                        </h2>
                        <br />
                        <p>
                            Siusplau, introdueixi de nou el seu nom d`usuari.
                            Enviarem un correu per a poder recuperar la paraula
                            de pas.
                        </p>
                        <br />
                        <Form
                            noValidate
                            validated={formValidated}
                            onSubmit={handleSubmit}
                        >
                            <UserName
                                id={"email"}
                                name={"email"}
                                changeInput={changeInput}
                                email={email}
                            />
                            <Form.Row className="mt-4 d-flex justify-content-center">
                                <Button type="submit">Continuar</Button>
                            </Form.Row>
                        </Form>
                    </div>
                </div>
                <ModalClose
                    centered
                    setshow={setshow}
                    show={show}
                    title={"Recuperació paraula de pas"}
                    text={
                        "S'ha enviat un correu a la direcció introduïda amb les instruccions per a recuperar la paraula de pas"
                    }
                    route="/"
                />
                <ModalClose
                    centered
                    setshow={setshowUserNotFound}
                    show={showUserNotFound}
                    title={"Recuperació paraula de pas"}
                    text={"El correu introduït no correspon a cap usuari"}
                />
            </div>
        </>
    );
}
