import React from "react";
import dynamic from "next/dynamic";
import { useRouter } from "next/router";
import { useFetch } from "../Hooks/useFetch";
import Spinner from "../components/globals/Spinner";
import { colors } from "../styles/theme";

//Carrega el mapa sense el SSR de NextJS
const Map = dynamic(() => import("../components/main/Mapa/HistoricMap"), {
    ssr: false,
});

const Main = () => {
    const router = useRouter();
    const { token, id, minutes, lat, lng, zoom } = router.query;

    let url;
    if (id) {
        url = `/api/data/historic?${new URLSearchParams({
            id,
            minutes,
        }).toString()}`;
    }

    const { data, loading, error } = useFetch({ url, token });

    if (minutes > 1440) return <h2>No es poden mostrar més de 24h de cop</h2>;

    return (
        <>
            <section className="sectioncontainer">
                {!loading && data && (
                    <Map data={data} lat={lat} lng={lng} zoom={zoom} />
                )}
                {loading && (
                    <div
                        id="spinner"
                        style={{ visibility: "visible" }}
                        className="spinnerDiv"
                    >
                        <Spinner colorspinner={colors.primaryBlue} />
                    </div>
                )}
                {error && (
                    <h4>
                        Alguna cosa ha fallat, no es poden carregar les dades,
                        recarregui
                    </h4>
                )}
            </section>
            <style jsx>{`
                .sectioncontainer {
                    display: grid;
                    width: 100vw;
                    height: 100vh;
                }
            `}</style>
        </>
    );
};

export default Main;
