import mongoose from "mongoose";

const { MONGODB_URI, MONGODB_DB } = process.env;

//http://portquiz.net:27017/

if (!MONGODB_URI) {
    throw new Error(
        "Please define the MONGODB_URI environment variable inside .env.local"
    );
}

if (!MONGODB_DB) {
    throw new Error(
        "Please define the MONGODB_DB environment variable inside .env.local"
    );
}

/**
 * Global se usa aquí para mantener una conexión en caché a través de recargas en caliente
 * en desarrollo. Esto evita que las conexiones crezcan exponencialmente durante el uso
 * de la ruta API.
 */
let cached = global.mongo;

if (!cached) {
    cached = global.mongo = { conn: null, promise: null };
}

export async function connectToDatabase() {
    if (cached.conn) {
        return cached.conn;
    }

    if (!cached.promise) {
        const opts = {
            useNewUrlParser: true,
            useCreateIndex: true,
            useUnifiedTopology: true,
            bufferCommands: false,
            bufferMaxEntries: 0,
            useFindAndModify: false,
            //add
            autoIndex: true,
            keepAlive: true,
        };

        cached.promise = mongoose.connect(MONGODB_URI, opts).then((mongo) => {
            console.log("Connect to MongoDB Atlas")
            return mongo;
        });
    }
    cached.conn = await cached.promise;

    return cached.conn;
}
