/**
 * Middleware que controla que les peticions a certs Endpoints
 * del servidor només les puguin fer usuaris autentificats.
 */

import Account from "../../models/Account";
import { returnUserByToken } from "../../pages/api/functions";

export function withAuth(handler) {   
    return async (req, res) => {
        const token = req?.headers?.token;

        //Sino token no autoritzat
        if (!token) {
            return res
                .status(401)
                .send({ message: "Not authenticated. No token header" });
        }

        try {
            const user = await returnUserByToken(token);       
            //Token no vàlid
            if (!user?.isTokenOK) {
                return res.status(401).end("Token not valid");
            }
            //No user no autoritzat
            if (!user?.user_id) {
                return res.status(401).end("Not authenticated");
            }

            //Pass user id into req
            req.user_id = user?.user_id;
        } catch (error) {
            console.log(error.errorInfo);
            const errorCode = error.errorInfo.code;
            error.status = 401;
            if (errorCode === "auth/internal-error") {
                error.status = 500;
            }
            //return errors
            return res.status(error.status).json({ error: errorCode });
        }

        return handler(req, res);
    };
}

export function withRoles(handler, ...rol) {
    return async (req, res) => {
        const account = await Account.findById(req.user_id);

        //Si no té true el rol o no es troba el compte forbbiden
        if (!account[rol] || !account) {
            return res.status(403).json({
                success: false,
                message: "No tens permís per a realitzar aquesta acció",
            });
        }
        return handler(req, res);
    };
}
